       subroutine bcui3d(x,y,z,f,xi,yi,zi,fi,fxi,fyi,fzi,
     & fxxi,fyyi,fzzi,fxyi,fxzi,fyzi)
	  
c***********************************************************************
c      Barycentric Cubic Interpolation 3D
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Gambelas, Mon Feb 29 14:31:32 WET 2016
c      Obrigado Rogerio!
c***********************************************************************

       integer i,j,k
       
       real*8 f(4,4,4),a(4,4,4)
       real*8 x(4),y(4),z(4)
       real*8 px(4),py(4),pz(4)
       real*8 sx(4),sy(4),sz(4)
       real*8 qx(4),qy(4),qz(4)
       real*8 xi,x1,x2,x3,x4
       real*8 yi,y1,y2,y3,y4
       real*8 zi,z1,z2,z3,z4
       real*8 fi,fxi,fyi,fzi
       real*8 fxxi,fyyi,fzzi
       real*8 fxyi,fxzi,fyzi

       x1 = x(1)
       x2 = x(2)
       x3 = x(3)
       x4 = x(4)
       
       y1 = y(1)
       y2 = y(2)
       y3 = y(3)
       y4 = y(4)
       
       z1 = z(1)
       z2 = z(2)
       z3 = z(3)
       z4 = z(4)
       
       px(1) = (x1-x2)*(x1-x3)*(x1-x4)
       px(2) = (x2-x1)*(x2-x3)*(x2-x4)
       px(3) = (x3-x1)*(x3-x2)*(x3-x4)
       px(4) = (x4-x1)*(x4-x2)*(x4-x3)
       
       py(1) = (y1-y2)*(y1-y3)*(y1-y4)
       py(2) = (y2-y1)*(y2-y3)*(y2-y4)
       py(3) = (y3-y1)*(y3-y2)*(y3-y4)
       py(4) = (y4-y1)*(y4-y2)*(y4-y3)
       
       pz(1) = (z1-z2)*(z1-z3)*(z1-z4)
       pz(2) = (z2-z1)*(z2-z3)*(z2-z4)
       pz(3) = (z3-z1)*(z3-z2)*(z3-z4)
       pz(4) = (z4-z1)*(z4-z2)*(z4-z3)
       
c      Yes, a(1,1,1) is zero, but using the do loops 
c      shortens the code. 

       do k = 1,4
       do j = 1,4
       
          do i = 1,4
       
             a(k,j,i) = (f(k,j,i) - f(1,1,1))/( pz(k)*py(j)*px(i) )

          end do

       end do
       end do
       
       px(1) = (xi-x2)*(xi-x3)*(xi-x4)
       px(2) = (xi-x1)*(xi-x3)*(xi-x4)
       px(3) = (xi-x1)*(xi-x2)*(xi-x4)
       px(4) = (xi-x1)*(xi-x2)*(xi-x3)
       
       py(1) = (yi-y2)*(yi-y3)*(yi-y4)
       py(2) = (yi-y1)*(yi-y3)*(yi-y4)
       py(3) = (yi-y1)*(yi-y2)*(yi-y4)
       py(4) = (yi-y1)*(yi-y2)*(yi-y3)
       
       pz(1) = (zi-z2)*(zi-z3)*(zi-z4)
       pz(2) = (zi-z1)*(zi-z3)*(zi-z4)
       pz(3) = (zi-z1)*(zi-z2)*(zi-z4)
       pz(4) = (zi-z1)*(zi-z2)*(zi-z3)
       
       qx(1) = (xi-x2)*(xi-x3) + (xi-x2)*(xi-x4) + (xi-x3)*(xi-x4)
       qx(2) = (xi-x1)*(xi-x3) + (xi-x1)*(xi-x4) + (xi-x3)*(xi-x4)
       qx(3) = (xi-x1)*(xi-x2) + (xi-x1)*(xi-x4) + (xi-x2)*(xi-x4)
       qx(4) = (xi-x1)*(xi-x2) + (xi-x1)*(xi-x3) + (xi-x2)*(xi-x3)
       
       qy(1) = (yi-y2)*(yi-y3) + (yi-y2)*(yi-y4) + (yi-y3)*(yi-y4)
       qy(2) = (yi-y1)*(yi-y3) + (yi-y1)*(yi-y4) + (yi-y3)*(yi-y4)
       qy(3) = (yi-y1)*(yi-y2) + (yi-y1)*(yi-y4) + (yi-y2)*(yi-y4)
       qy(4) = (yi-y1)*(yi-y2) + (yi-y1)*(yi-y3) + (yi-y2)*(yi-y3)
       
       qz(1) = (zi-z2)*(zi-z3) + (zi-z2)*(zi-z4) + (zi-z3)*(zi-z4)
       qz(2) = (zi-z1)*(zi-z3) + (zi-z1)*(zi-z4) + (zi-z3)*(zi-z4)
       qz(3) = (zi-z1)*(zi-z2) + (zi-z1)*(zi-z4) + (zi-z2)*(zi-z4)
       qz(4) = (zi-z1)*(zi-z2) + (zi-z1)*(zi-z3) + (zi-z2)*(zi-z3)
       
       sx(1) = 2.0*( 3.0*xi - x2 - x3 - x4 )
       sx(2) = 2.0*( 3.0*xi - x1 - x3 - x4 )
       sx(3) = 2.0*( 3.0*xi - x1 - x2 - x4 )
       sx(4) = 2.0*( 3.0*xi - x1 - x2 - x3 )
       
       sy(1) = 2.0*( 3.0*yi - y2 - y3 - y4 )
       sy(2) = 2.0*( 3.0*yi - y1 - y3 - y4 )
       sy(3) = 2.0*( 3.0*yi - y1 - y2 - y4 )
       sy(4) = 2.0*( 3.0*yi - y1 - y2 - y3 )
       
       sz(1) = 2.0*( 3.0*zi - z2 - z3 - z4 )
       sz(2) = 2.0*( 3.0*zi - z1 - z3 - z4 )
       sz(3) = 2.0*( 3.0*zi - z1 - z2 - z4 )
       sz(4) = 2.0*( 3.0*zi - z1 - z2 - z3 )
       
       fi   = f(1,1,1)
       fxi  = 0.0
       fyi  = 0.0
       fzi  = 0.0
       fxxi = 0.0
       fyyi = 0.0
       fzzi = 0.0
       fxyi = 0.0
       fxzi = 0.0
       fyzi = 0.0

c      No patience for third order derivatives...

       do k = 1,4
       do j = 1,4
       
          do i = 1,4
       
              fi  =   fi + a(k,j,i)*px(i)*py(j)*pz(k)
             fxi  =  fxi + a(k,j,i)*qx(i)*py(j)*pz(k)
            fxxi  = fxxi + a(k,j,i)*sx(i)*py(j)*pz(k)
	     fyi  =  fyi + a(k,j,i)*px(i)*qy(j)*pz(k)
	    fyyi  = fyyi + a(k,j,i)*px(i)*sy(j)*pz(k)
	     fzi  =  fzi + a(k,j,i)*px(i)*py(j)*qz(k)
	    fzzi  = fzzi + a(k,j,i)*px(i)*py(j)*sz(k)
            fxyi  = fxyi + a(k,j,i)*qx(i)*qy(j)*pz(k)
	    fxzi  = fxzi + a(k,j,i)*qx(i)*py(j)*qz(k)
	    fyzi  = fyzi + a(k,j,i)*px(i)*qy(j)*qz(k)
	    
          end do

       end do
       end do

c***********************************************************************
c     Back to main:
c***********************************************************************
       
       return 
       
       end
       
