       subroutine bpai2d(x,y,f,xi,yi,fi,fxi,fyi,fxxi,fyyi,fxyi)
	  
c***********************************************************************
c      Barycentric Parabolic Interpolation 2D
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Gambelas, Tue Jun 14 21:30:00 WEST 2011
c***********************************************************************

       real*8 f(3,3)
       real*8 x(3),y(3)
       real*8 px(3),py(3),sx(3),sy(3)
       real*8 fi,fxi,fyi,fxxi,fyyi,fxyi
       real*8 xi,x1,x2,x3
       real*8 yi,y1,y2,y3
       real*8     a12,a13
       real*8 a21,a22,a23
       real*8 a31,a32,a33

       x1 = x(1)
       x2 = x(2)
       x3 = x(3)
       
       y1 = y(1)
       y2 = y(2)
       y3 = y(3)

       px(1) = (x1-x2)*(x1-x2)
       px(2) = (x2-x1)*(x2-x3)
       px(3) = (x3-x1)*(x3-x2)

       py(1) = (y1-y2)*(y1-y3)
       py(2) = (y2-y1)*(y2-y3)
       py(3) = (y3-y1)*(y3-y2)
       
       a12 = (f(1,2)-f(1,1))/( px(2)*py(1) )
       a13 = (f(1,3)-f(1,1))/( px(3)*py(1) )
       a21 = (f(2,1)-f(1,1))/( px(1)*py(2) )
       a22 = (f(2,2)-f(1,1))/( px(2)*py(2) )
       a23 = (f(2,3)-f(1,1))/( px(3)*py(2) )
       a31 = (f(3,1)-f(1,1))/( px(1)*py(3) )
       a32 = (f(3,2)-f(1,1))/( px(2)*py(3) )
       a33 = (f(3,3)-f(1,1))/( px(3)*py(3) )

       px(1) = (xi-x2)*(xi-x2)
       px(2) = (xi-x1)*(xi-x3)
       px(3) = (xi-x1)*(xi-x2)

       py(1) = (yi-y2)*(yi-y3)
       py(2) = (yi-y1)*(yi-y3)
       py(3) = (yi-y1)*(yi-y2)
       
       sx(1) = 2*xi - x2 - x3
       sx(2) = 2*xi - x1 - x3
       sx(3) = 2*xi - x1 - x2
       
       sy(1) = 2*yi - y2 - y3
       sy(2) = 2*yi - y1 - y3
       sy(3) = 2*yi - y1 - y2

       fi = f(1,1) + ( a12*px(2) + a13*px(3) )*py(1) +
     & ( a21*px(1) +   a22*px(2) + a23*px(3) )*py(2) +
     & ( a31*px(1) +   a32*px(2) + a33*px(3) )*py(3)
     
       fxi =         ( a12*sx(2) + a13*sx(3) )*py(1) +
     & ( a21*sx(1) +   a22*sx(2) + a23*sx(3) )*py(2) +
     & ( a31*sx(1) +   a32*sx(2) + a33*sx(3) )*py(3)
       
       fyi =         ( a12*px(2) + a13*px(3) )*sy(1) +
     & ( a21*px(1) +   a22*px(2) + a23*px(3) )*sy(2) +
     & ( a31*px(1) +   a32*px(2) + a33*px(3) )*sy(3)
     
       fxxi = ( a12 + a13 )*py(1) +
     &  ( a21 + a22 + a23 )*py(2) +
     &  ( a31 + a32 + a33 )*py(3)
     
       fyyi = ( a21 + a31 )*px(1) +
     &  ( a12 + a22 + a32 )*px(2) +
     &  ( a13 + a23 + a33 )*px(3)
     
       fxyi =      ( a12*sx(2) + a13*sx(3) )*sy(1) +
     & ( a21*sx(1) + a22*sx(2) + a23*sx(3) )*sy(2) +
     & ( a31*sx(1) + a32*sx(2) + a33*sx(3) )*sy(3)

c***********************************************************************
c     Back to main:
c***********************************************************************
       
       return 
       
       end

