       subroutine bpai3d(x,y,z,f,xi,yi,zi,fi,fxi,fyi,fzi,
     & fxxi,fyyi,fzzi,fxyi,fxzi,fyzi)
	  
c***********************************************************************
c      Barycentric Parabolic Interpolation 3D
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Faro, Wed Jun 15 22:30:00 WEST 2011
c***********************************************************************
       
       integer i,j,k
       
       real*8 f(3,3,3),a(3,3,3)
       real*8 x(3),y(3),z(3)
       real*8 px(3),py(3),pz(3)
       real*8 sx(3),sy(3),sz(3)
       real*8 fi,fxi,fyi,fzi,fxxi,fyyi,fzzi,fxyi,fxzi,fyzi
       real*8 xi,x1,x2,x3
       real*8 yi,y1,y2,y3
       real*8 zi,z1,z2,z3
             
       x1 = x(1)
       x2 = x(2)
       x3 = x(3)
       
       y1 = y(1)
       y2 = y(2)
       y3 = y(3)
       
       z1 = z(1)
       z2 = z(2)
       z3 = z(3)

       px(1) = (x1-x2)*(x1-x3)
       px(2) = (x2-x1)*(x2-x3)
       px(3) = (x3-x1)*(x3-x2)
       
       py(1) = (y1-y2)*(y1-y3)
       py(2) = (y2-y1)*(y2-y3)
       py(3) = (y3-y1)*(y3-y2)
       
       pz(1) = (z1-z2)*(z1-z3)
       pz(2) = (z2-z1)*(z2-z3)
       pz(3) = (z3-z1)*(z3-z2)

c      Yes, a(1,1,1) is zero, but using the do loops 
c      shortens the code. 

       do i = 1,3
       
          do j = 1,3
          
             do k = 1,3
       
                a(i,j,k) = (f(i,j,k) - f(1,1,1))/( px(k)*py(j)*pz(i) )
             
             end do 
             
          end do
       
       end do

       px(1) = (xi-x2)*(xi-x3)
       px(2) = (xi-x1)*(xi-x3)
       px(3) = (xi-x1)*(xi-x2)
       
       py(1) = (yi-y2)*(yi-y3)
       py(2) = (yi-y1)*(yi-y3)
       py(3) = (yi-y1)*(yi-y2)
       
       pz(1) = (zi-z2)*(zi-z3)
       pz(2) = (zi-z1)*(zi-z3)
       pz(3) = (zi-z1)*(zi-z2)

       sx(1) = 2*xi - x2 -x3
       sx(2) = 2*xi - x1 -x3
       sx(3) = 2*xi - x1 -x2
       
       sy(1) = 2*yi - y2 -y3
       sy(2) = 2*yi - y1 -y3
       sy(3) = 2*yi - y1 -y2
       
       sz(1) = 2*zi - z2 -z3
       sz(2) = 2*zi - z1 -z3
       sz(3) = 2*zi - z1 -z2

       fi   = f(1,1,1)
       fxi  = 0.0
       fyi  = 0.0
       fzi  = 0.0
       fxxi = 0.0
       fyyi = 0.0
       fzzi = 0.0
       fxyi = 0.0
       fxzi = 0.0
       fyzi = 0.0

       do i = 1,3

          do j = 1,3

             do k = 1,3

             fi   = fi   + a(i,j,k)*px(k)*py(j)*pz(i)
             fxi  = fxi  + a(i,j,k)*sx(k)*py(j)*pz(i)
             fyi  = fyi  + a(i,j,k)*px(k)*sy(j)*pz(i)
             fzi  = fzi  + a(i,j,k)*px(k)*py(j)*sz(i)
             fxxi = fxxi + a(i,j,k)* 2.0 *py(j)*pz(i)
             fyyi = fyyi + a(i,j,k)*px(k)* 2.0 *pz(i)
             fzzi = fzzi + a(i,j,k)*px(k)*py(j)* 2.0
             fxyi = fxyi + a(i,j,k)*sx(k)*sy(j)*pz(i)
             fxzi = fxzi + a(i,j,k)*sx(k)*py(j)*sz(i)
             fyzi = fyzi + a(i,j,k)*px(k)*sy(j)*sz(i)

             end do

          end do

       end do

c***********************************************************************
c     Back to main:
c***********************************************************************

       return

       end
