       subroutine calpav(ctitle,nthtas,nphi,iad)

c***********************************************************************
c      CALPAV.FOR
c      CALculation of Pressure And particle Velocity
c      Written by Orlando Camargo Rodriguez
c      Faro, Tue Jul 22 23:17:00 WEST 2014
c***********************************************************************

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       character(len=128) header
       character(len=60 ) ctitle
       character(len=8  ) v1

       integer*4 irefl(np)
       integer*4 kseg(nrefrx+nrefl+2),kaux(nrefrx+nreflx+2)

       integer*4 i,iad,ii,imax
       integer*4 j,k,l,ll,lvar
       integer*4 nthtas,nphi,na
       integer*4 mrows,ncols

       real*8 r1(3),r2(3),r3(3)
       real*8 xp(3)
       real*8 thetai,ctheta,phii
       real*8 cc,sigmai,cxi,cyi,czi,sxi,syi,szi
       real*8 cxxi,cyyi,czzi,cxyi,cxzi,cyzi
       real*8 delta
       real*8 dummy
       
       complex*16 dpdx(np),dpdy(np),dpdz(np)
       complex*16 zp(3)
       
       complex*16 p1,p2,p3
       complex*16 zpi,zpxi

c***********************************************************************

       call csvals(xx,yx,zx,cx,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
      
      omega  = 2.0*pi*freqx
      lambda =     cx/freqx
      
      dtheta = dtheta*pi/180.0
      delta  =    lambda/10.0

      na = 0
      
      kseg(1) = 1
    
c***********************************************************************
c    Start writing the OUTFIL:
c***********************************************************************

c     96 characters: 

      header = 'Signal Processing LABoratory: TRACEO3D
     & acoustic ray tracing program - Particle VeLocity'
     
      header(117:124) = char(ichar(' '))
      header(125:126) = char(0) // char(1)
       
      write(header(127:128),'(a)') int(19785,kind=2)

      open(unit=outfil,file='pvl.mat',form='unformatted',
     & access='stream')
     
c     Open the file and write the header:

      do i = 1, 16
         j = 8*(i-1) + 1
         write(outfil) header(j:j+7)
      end do

c     Write the angles:

      v1 = 'ray_thts'

      call dtelrv(outfil,1,nthtas,8,v1,thetas)

      v1 = 'ray_phis'
	   
      call dtelrv(outfil,1,nphi,8,v1,phi)

c     Write the array coordinates:

      if ((nxa*nya*nza).eq.0) then
      na = max(nxa,nya,nza)
      end if

      v1 = 'xarray3d'

      if (na.ne.0) then
      ncols = na
      else
      ncols = nxa
      end if
      
      call dtelrv(outfil,1,ncols,8,v1,xarray)

      v1 = 'yarray3d'

      if (na.ne.0) then
      ncols = na
      else
      ncols = nya
      end if
    
      call dtelrv(outfil,1,ncols,8,v1,yarray)

      v1 = 'zarray3d'

      if (na.ne.0) then
      ncols = na
      else
      ncols = nza
      end if

      call dtelrv(outfil,1,ncols,8,v1,zarray)

c***********************************************************************
c      Solve the EIKonal set of EQuations,
c      Solve the DYNamic set of Equations,
c      Get ray pressure:
c***********************************************************************
       
       do ii = 1,nphi
       
          phii = phi(ii)*pi/180.0
       
       do i  = 1,nthtas

           thetai = -thetas(i)*pi/180.0

           ctheta = abs( cos( thetai ) )

c          Trace a ray as long as it is neither 90 or -90:

           if (ctheta.gt.1.0e-7) then

           call seikeq(thetai,phii,imax,irefl)
	   call sdyneq(thetai,phii,imax,irefl)

c=======================================================================
c=======================================================================
c          Get indexes of ray segments:
c=======================================================================
c=======================================================================

c	   No refractions/no reflections:

	   if ((nrefl.eq.0).and.(nrefr.eq.0)) then
	
	   kseg(2) = imax
	   nseg    = 2
c-----------------------------------------------------------------------
c	   No refractions/some reflections:

	   elseif ((nrefl.ne.0).and.(nrefr.eq.0)) then

	   do k = 1,nrefl

	      kseg(k+1) = krefl(k)

	   end do

	   if (krefl(nrefl).lt.imax) then
	   kseg(nrefl+2) = imax
	   nseg = nrefl + 2
	   else
	   nseg = nrefl + 1
	   end if
c-----------------------------------------------------------------------
c	   Some refractions/no reflections:

	   elseif ((nrefl.eq.0).and.(nrefr.ne.0)) then

	   do k = 1,nrefr

	      kseg(k+1) = krefr(k)
	
	   end do
	
	   if (krefr(nrefr).lt.imax) then
	   kseg(nrefr+2) = imax
	   nseg = nrefr + 2
	   else
	   nseg = nrefr + 1
	   end if
c-----------------------------------------------------------------------
c	   Some refractions/some reflections:

	   else
	
	   kaux(1) = 1
	
	   do k = 1,nrefl

	      kaux(k+1) = krefl(k)
	
	   end do

	   do k = 1,nrefr

	      kaux(nrefl+k+1) = krefr(k)
	
	   end do
	
	   nseg = nrefl+nrefr+1

	   call sorti(nseg,kaux,kseg)

	   if (kseg(nseg).lt.imax) then
	   nseg = nseg + 1
	   kseg(nseg) = imax
	   end if
	
c-----------------------------------------------------------------------
           end if
c	   if ((nrefl.eq.0).and.(nrefr.eq.0)) then

c=======================================================================
c=======================================================================
c          Normal search:
c=======================================================================
c=======================================================================

c          Linear array:

	   if (iad.eq.1) then

c***********************************************************************

 	      do l = 1,na

	      r2(1) = xarray(l)
	      r2(2) = yarray(l)
	      r2(3) = zarray(l)

	      r1(1) = r2(1) - delta
	      r1(2) = r2(2)
	      r1(3) = r2(3)
	      
	      r3(1) = r2(1) + delta
	      r3(2) = r2(2)
	      r3(3) = r2(3)

	      call gpress(imax,r1,p1,nseg,kseg)
	      call gpress(imax,r2,p2,nseg,kseg)
	      call gpress(imax,r3,p3,nseg,kseg)
	      
	      pres1d(l) = pres1d(l) + p2
	      
	      xp(1) = r1(1)
	      xp(2) = r2(1)
	      xp(3) = r3(1)

	      zp(1) = p1
	      zp(2) = p2
	      zp(3) = p3
	      
	      call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	      
	      dpdx(l) = dpdx(l) - imunit*zpxi
	      
	      r1(1) = r2(1)
	      r1(2) = r2(2) - delta
	      r1(3) = r2(3)
	      
	      r3(1) = r2(1)
	      r3(2) = r2(2) + delta
	      r3(3) = r2(3)
	      
	      call gpress(imax,r1,p1,nseg,kseg)
	      call gpress(imax,r3,p3,nseg,kseg)
	      
	      xp(1) = r1(2)
	      xp(2) = r2(2)
	      xp(3) = r3(2)
              
	      zp(1) = p1
	      zp(3) = p3
	      
	      call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	      
	      dpdy(l) = dpdy(l) - imunit*zpxi
	      
	      r1(1) = r2(1)
	      r1(2) = r2(2)
	      r1(3) = r2(3) - delta
	      
	      r3(1) = r2(1)
	      r3(2) = r2(2)
	      r3(3) = r2(3) + delta

	      call gpress(imax,r1,p1,nseg,kseg)
	      call gpress(imax,r3,p3,nseg,kseg)
	      
	      xp(1) = r1(3)
	      xp(2) = r2(3)
	      xp(3) = r3(3)
              
	      zp(1) = p1
	      zp(3) = p3
	      
	      call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	      
	      dpdz(l) = dpdz(l) - imunit*zpxi
	      
	      end do

c***********************************************************************
c          Plane array:

           else

              ll = 0

              do j = 1,nxa
	         
		 r2(1) = xarray(j)
		 
	      do k = 1,nya
	      
	         r2(2) = yarray(k)

	      do l = 1,nza
	      
	         r2(3) = zarray(l)
		 
		 ll = ll + 1
		 
		 r1(1) = r2(1) - delta
		 r1(2) = r2(2)
		 r1(3) = r2(3)
	
		 r3(1) = r2(1) + delta
		 r3(2) = r2(2)
		 r3(3) = r2(3)
	      
	      	 call gpress(imax,r1,p1,nseg,kseg)
	      	 call gpress(imax,r2,p2,nseg,kseg)
	      	 call gpress(imax,r3,p3,nseg,kseg)
		 
		 pres1d(ll) = pres1d(ll) + p2
		 
		 xp(1) = r1(1)
		 xp(2) = r2(1)
		 xp(3) = r3(1)

		 zp(1) = p1
		 zp(2) = p2
		 zp(3) = p3
	      
		 call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	
		 dpdx(ll) = dpdx(ll) - imunit*zpxi
	
		 r1(1) = r2(1)
		 r1(2) = r2(2) - delta
		 r1(3) = r2(3)
	
		 r3(1) = r2(1)
		 r3(2) = r2(2) + delta
		 r3(3) = r2(3)
	      
		 call gpress(imax,r1,p1,nseg,kseg)
		 call gpress(imax,r3,p3,nseg,kseg)
	
		 xp(1) = r1(2)
		 xp(2) = r2(2)
		 xp(3) = r3(2)
 
		 zp(1) = p1
		 zp(3) = p3
	      
	         call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	      
	         dpdy(ll) = dpdy(ll) - imunit*zpxi
	      
		 r1(1) = r2(1)
		 r1(2) = r2(2)
		 r1(3) = r2(3) - delta
	
		 r3(1) = r2(1)
		 r3(2) = r2(2)
		 r3(3) = r2(3) + delta

		 call gpress(imax,r1,p1,nseg,kseg)
		 call gpress(imax,r3,p3,nseg,kseg)
	
		 xp(1) = r1(3)
		 xp(2) = r2(3)
		 xp(3) = r3(3)
 
		 zp(1) = p1
		 zp(3) = p3
	
		 call cbpa1d(xp,zp,xp(2),zpi,zpxi)
	
		 dpdz(ll) = dpdz(ll) - imunit*zpxi

	      end do
	      end do
	      end do

c***********************************************************************

	   end if
c	   if (iad.eq.1) then

c=======================================================================
c=======================================================================
c          End of calculations
c=======================================================================
c=======================================================================

c***********************************************************************
c          if (ctheta.gt.1.0e-7) then
           end if

       end do
       
       end do
       
c***********************************************************************
c     Write the particle velocity components:
c***********************************************************************

      lvar = 8
      
c=======================================================================
c     Linear array:
      
      if (iad.eq.1) then

	 ncols = 1

	 v1 = 'pressure'
	 
         call dtelcv(outfil,na,ncols,lvar,v1,pres1d)
	 
         v1 = 'dp_dx_3d'

         call dtelcv(outfil,na,ncols,lvar,v1,dpdx)

         v1 = 'dp_dy_3d'
	 
	 call dtelcv(outfil,na,ncols,lvar,v1,dpdy)
	 
         v1 = 'dp_dz_3d'

	 call dtelcv(outfil,na,ncols,lvar,v1,dpdz)

c=======================================================================
c     Plane array:
      
      else
      
         if (nxa.eq.1) then
	  
	  ncols = nya
	  mrows = nza

	  elseif (nya.eq.1) then
	  
	  ncols = nxa
	  mrows = nza
	  
	  else
	  
	  ncols = nxa
	  mrows = nya
	  
	  end if

	  v1 = 'pressure'
	
 	  call dtelcv(outfil,mrows,ncols,lvar,v1,pres1d)
	  
 	  v1 = 'dp_dx_3d'
	
 	  call dtelcv(outfil,mrows,ncols,lvar,v1,dpdx)

 	  v1 = 'dp_dy_3d'

	  call dtelcv(outfil,mrows,ncols,lvar,v1,dpdy)
	
 	  v1 = 'dp_dz_3d'

	  call dtelcv(outfil,mrows,ncols,lvar,v1,dpdz)
	  
c     if (iad.eq.1) then
      end if

c***********************************************************************

5000   format(i6)

c***********************************************************************
c      Close the OUTFIL:
c***********************************************************************

       close(outfil)

c***********************************************************************
c     Back to main:
c***********************************************************************

      return
      
      end
