       subroutine csvals(xi,yi,zi,ci,cc,si,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

c**********************************************************************
c      Sound speed and slowness interpolation subroutine
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Mexilhoeira Grande, Sun Jun 12 13:00:00 WEST 2011
c****&******************************************************************
c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       real*8 xi,yi,zi,ci,si,cc
       real*8 cxi,cyi,czi,sxi,syi,szi
       real*8 cxxi,cyyi,czzi
       real*8 cxyi,cxzi,cyzi
       real*8 k,root,root32,root52,a
       real*8 bmunk,bmunk2,epsiln,eta

c-----------------------------------------------------------------------

       epsiln = 7.4e-3;
       bmunk  = 1300.0
       bmunk2 = bmunk*bmunk

c Initial values (to be changed at every case)
       
        cxi =  0.0
        cyi =  0.0
        czi =  0.0
       cxxi =  0.0
       cyyi =  0.0
       czzi =  0.0
       cxyi =  0.0
       cxzi =  0.0
       cyzi =  0.0

c=======================================================================
c       Analytical sound speed profiles:
c=======================================================================
	    if (cdist.eq.'ISOV') then

	         ci = c0(1)

	    elseif (cdist.eq.'LINP') then

	        k = (c0(2)-c0(1))/(z0(2)-z0(1))
	       ci = c0(1) + k*( zi - z0(1) )

	    elseif (cdist.eq.'PARP') then

	        k = (c0(2)-c0(1))/( (z0(2)-z0(1))**2.0 )
	       ci = c0(1) + k*( zi - z0(1) )**2.0
	      czi = 2*k*( zi - z0(1) )
	     czzi = 2*k

	    elseif (cdist.eq.'EXPP') then

	        k = log( c0(1)/c0(2) )/( z0(2) - z0(1) )
	       ci = c0(1)*exp( -k*( zi - z0(1) ) )
	      czi = -k*ci
	     czzi = k*k*ci

	    elseif (cdist.eq.'N2LP') then

	        k = ( ( c0(1)/c0(2) )**2.0 - 1 )/( z0(2) - z0(1) )
	     root = sqrt( 1 + k*( zi - z0(1) ) )
	   root32 = root**( 3.0/2.0 )
	   root52 = root**( 5.0/2.0 )  
	       ci = c0(1)/sqrt( 1 + k*( zi - z0(1) ) )
	      czi =    -k*c0(1)/( 2*root32 )
	     czzi = 3*k*k*c0(1)/( 4*root52 )

	    elseif (cdist.eq.'ISQP') then

	        a = ( c0(2)/c0(1) -1 )**2.0
	     root = sqrt( a/(1-a) )
	        k = root/( z0(2) - z0(1) )
	     root = sqrt( 1 + ( k*( zi - z0(1) ) )**(2.0) )
	   root32 = root**( 3.0/2.0 )
	   root52 = root**( 5.0/2.0 )
	       ci = c0(1)*( 1 + k*( zi - z0(1) )/root )
	      czi = c0(1)*k/root32
	     czzi = -3*c0(1)*k**3.0/root52

	    elseif (cdist.eq.'MUNK') then 

	    eta = 2*( zi - z0(1) )/bmunk
	     ci = c0(1)*( 1 + epsiln*( eta + exp(-eta) - 1 ) )
	    czi = 2*epsiln*c0(1)*( 1 - exp(-eta) )/bmunk
	   czzi = 4*epsiln*c0(1)*exp( -eta )/bmunk2

c=======================================================================
c       Tabulated sound speed profiles:
c=======================================================================

	    elseif (cdist.eq.'CX00') then

	    call cvals1(nx0,x0,c0,xi,ci,cxi,cxxi)
	    
	    elseif (cdist.eq.'C0Y0') then

	    call cvals1(ny0,y0,c0,yi,ci,cyi,cyyi)

	    elseif (cdist.eq.'C00Z') then

	    call cvals1(nz0,z0,c0,zi,ci,czi,czzi)

c=======================================================================
c       Tabulated sound speed fields:
c=======================================================================

	    elseif (cdist.eq.'CXY0') then
	    
	    call cvals2(nx0,ny0,x0,y0,c02d,xi,yi,
     &      ci,cxi,cyi,cxxi,cyyi,cxyi)

	    elseif (cdist.eq.'CX0Z') then
	    
	    call cvals2(nx0,nz0,x0,z0,c02d,xi,zi,
     &      ci,cxi,czi,cxxi,czzi,cxzi)

	    elseif (cdist.eq.'C0YZ') then
	    
	    call cvals2(ny0,nz0,y0,z0,c02d,yi,zi,
     &      ci,cyi,czi,cyyi,czzi,cyzi)

c=======================================================================
c       3D sound speed field:
c=======================================================================

	    elseif (cdist.eq.'CXYZ') then
	    
	    call cvals3(nx0,ny0,nz0,x0,y0,z0,c03d,xi,yi,zi,
     &      ci,cxi,cyi,czi,cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
     
c=======================================================================
c       None of the previous? stop calculations:
c=======================================================================

	    else

            print *, 'Unknown sound speed profile type'
	    print *, 'aborting calculations...'
            stop

	    end if

c-----------------------------------------------------------------------

        cc = ci*ci

        si =  1.0/ci 
       sxi = -cxi/cc
       syi = -cyi/cc
       szi = -czi/cc

c***********************************************************************
c     Back to main:
c***********************************************************************

       return

       end 
