       subroutine gpray(ia,ib,rh,pray)
       
c***********************************************************************
c      Get Pressure from RAY along a ray segment
c      This subroutine is called only when the normal is known to exist
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Mexilhoeira Grande, Sex Ago  2 16:09:37 WEST 2019
c****&******************************************************************

c      This subroutine has a problem with the calculation of normals 
c      over ray segments, by skiping some and creating gaps in the field...

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       integer*4 ia,ib,ie,im
       integer*4 ntrls

       real*8 qn(2,2),iqn(2,2)
       real*8 rh(3),rn(3),normal(3)
       real*8 esa(3),esb(3),esm(3)
       real*8 eah(3),ebh(3),emh(3)
       real*8 e1(3),e2(3)
       real*8 n0,n1,n2,n1_w1,n2_w2,n12_w12,n21_w21
       real*8 phin1,phin2,phin12,phin21,phin
       real*8 pa,pb,pm
       real*8 cn,sn,taun
       real*8 detqn,ap_aq
       real*8 alpha,thetax,dx,dy,dz,dr,dsan,dsab
       real*8 cpi
       real*8 dummy
       
       complex*16 an,pray

c***********************************************************************

       pray = (0.0 , 0.0)
       
       ntrls = 0
       
       call thorpe(freqx,alpha)
       
c=======================================================================
       
       dx = x(2) - x(1)
       dy = y(2) - y(1)       
       dz = z(2) - z(1)
       dr = sqrt( dx*dx + dy*dy )
       
       thetax = atan2(dz,dr)

c=======================================================================

c      The normal is to be search through bisection:
       
c      Unitary tangent vector at point a:

       esa(1) = x(ia+1) - x(ia)
       esa(2) = y(ia+1) - y(ia)
       esa(3) = z(ia+1) - z(ia)
       
       call norm(esa,n0)
       
       esa(1) = esa(1)/n0
       esa(2) = esa(2)/n0
       esa(3) = esa(3)/n0
       
c      Unitary tangent vector at point b:

       esb(1) = x(ib) - x(ib-1)
       esb(2) = y(ib) - y(ib-1)
       esb(3) = z(ib) - z(ib-1)
       
       call norm(esb,n0)
       
       esb(1) = esb(1)/n0
       esb(2) = esb(2)/n0
       esb(3) = esb(3)/n0
       
c      Unitary vector from a to the hydrophone:

       eah(1) = rh(1) - x(ia)
       eah(2) = rh(2) - y(ia)
       eah(3) = rh(3) - z(ia)
       
       call norm(eah,n0)
       
       eah(1) = eah(1)/n0
       eah(2) = eah(2)/n0
       eah(3) = eah(3)/n0

c      Unitary vector from b to the hydrophone:

       ebh(1) = rh(1) - x(ib)
       ebh(2) = rh(2) - y(ib)
       ebh(3) = rh(3) - z(ib)
       
       call norm(ebh,n0)
       
       ebh(1) = ebh(1)/n0
       ebh(2) = ebh(2)/n0
       ebh(3) = ebh(3)/n0

c=======================================================================
c      Search:

500    im = (ia+ib)/2

       if ((ib-ia).gt.1) then

c         Unitary vector at point m:

 	  esm(1) = x(im+1) - x(im)
 	  esm(2) = y(im+1) - y(im)
 	  esm(3) = z(im+1) - z(im)
 
 	  call norm( esm, n0 )
 
 	  esm(1) = esm(1)/n0
 	  esm(2) = esm(2)/n0
 	  esm(3) = esm(3)/n0
       
c         Unitary vector from m to hydrophone:

 	  emh(1) = rh(1) - x(im)
 	  emh(2) = rh(2) - y(im)
 	  emh(3) = rh(3) - z(im)
 
 	  call norm( emh, n0 )
 
 	  emh(1) = emh(1)/n0
 	  emh(2) = emh(2)/n0
 	  emh(3) = emh(3)/n0

c      Check inner products in the intervals [a,m] and [m,b]:
       
 	  call dot( esa, eah, pa )
 	  call dot( esb, ebh, pb )
 	  call dot( esm, emh, pm )

c         This procedure is repeated until ib = ia + 1:

 	  if (pa*pm.gt.0.0) then
 	  ia  = im
 	  esa(1) = esm(1)
 	  esa(2) = esm(2)
 	  esa(3) = esm(3)
 	  elseif (pb*pm.gt.0.0) then
 	  ib  = im
 	  esb(1) = esm(1)
 	  esb(2) = esm(2)
 	  esb(3) = esm(3)
 	  else
	   ntrls = ntrls + 1
	   if (ntrls.gt.201) then
	   print *, 'Normal search failure!'
	   return
	   end if
 	  end if

c         Update vectors at a and b:

c-------------------------------------------

 	  esa(1) = x(ia+1) - x(ia)
 	  esa(2) = y(ia+1) - y(ia)
 	  esa(3) = z(ia+1) - z(ia)
 
 	  call norm( esa, n0 )
 
 	  esa(1) = esa(1)/n0
 	  esa(2) = esa(2)/n0
 	  esa(3) = esa(3)/n0
       
c-------------------------------------------

 	  esb(1) = x(ib)-x(ib-1)
 	  esb(2) = y(ib)-y(ib-1)
 	  esb(3) = z(ib)-z(ib-1)
 
 	  call norm( esb, n0 )
 
 	  esb(1) = esb(1)/n0
 	  esb(2) = esb(2)/n0
 	  esb(3) = esb(3)/n0
       
c-------------------------------------------

 	  eah(1) = rh(1) - x(ia)
 	  eah(2) = rh(2) - y(ia)
 	  eah(3) = rh(3) - z(ia)
 
 	  call norm( eah, n0 )
 
 	  eah(1) = eah(1)/n0
 	  eah(2) = eah(2)/n0
 	  eah(3) = eah(3)/n0
       
c-----------------------------------------------------------------------

 	  ebh(1) = rh(1) - x(ib)
 	  ebh(2) = rh(2) - y(ib)
 	  ebh(3) = rh(3) - z(ib)
 
 	  call norm( ebh, n0 )
 
 	  ebh(1) = ebh(1)/n0
 	  ebh(2) = ebh(2)/n0
 	  ebh(3) = ebh(3)/n0

          goto 500
       
      end if

c-----------------------------------------------------------------------
c      Bisection is finished, get the ray amplitude:
c-----------------------------------------------------------------------
c      Case 1, the normal is at a:

       if (pa.eq.0.0) then 
       
       qn(1,1) = q(ia,1,1)
       qn(1,2) = q(ia,1,2)
       qn(2,1) = q(ia,2,1)
       qn(2,2) = q(ia,2,2)
	  
       an = exp( imunit*( omega*tau(ia) - caustc(ia) ) )*amp(ia)

c-----------------------------------------------------------------------
c      Case 2, the normal is at b:

       elseif (pb.eq.0.0) then 
       
       qn(1,1) = q(ib,1,1)
       qn(1,2) = q(ib,1,2)
       qn(2,1) = q(ib,2,1)
       qn(2,2) = q(ib,2,2)
	  
       an = exp( imunit*( omega*tau(ib) - caustc(ib) ) )*amp(ib)

c-----------------------------------------------------------------------
c      Case 3, the normal is between a and b:
        
       else
       
       eah(1) = rh(1) - x(ia)
       eah(2) = rh(2) - y(ia)
       eah(3) = rh(3) - z(ia)
       
       call dot( esa, eah, pa )
	  
       rn(1) = x(ia) + pa*esa(1)
       rn(2) = y(ia) + pa*esa(2)
       rn(3) = z(ia) + pa*esa(3)
       
       dx = x(ib) - x(ia)
       dy = y(ib) - y(ia)       
       dz = z(ib) - z(ia)
           
       dsab = sqrt( dx*dx + dy*dy + dz*dz )
       
       dx = rn(1) - x(ia)
       dy = rn(2) - y(ia)       
       dz = rn(3) - z(ia)
          
       dsan = sqrt( dx*dx + dy*dy + dz*dz )

       taun =   tau(ia) + ( tau(ib)- tau(ia))*dsan/dsab
       sn    =    s(ia) +                     dsan
       cn    = cray(ia) + (cray(ib)-cray(ia))*dsan/dsab
	  
       qn(1,1) = q(ia,1,1) + (q(ib,1,1)-q(ia,1,1))*dsan/dsab
       qn(1,2) = q(ia,1,2) + (q(ib,1,2)-q(ia,1,2))*dsan/dsab
       qn(2,1) = q(ia,2,1) + (q(ib,2,1)-q(ia,2,1))*dsan/dsab
       qn(2,2) = q(ia,2,2) + (q(ib,2,2)-q(ia,2,2))*dsan/dsab

       detqn = qn(1,1)*qn(2,2)-qn(1,2)*qn(2,1)

       ap_aq = 1.0/abs(detqn)*cn/cx*cos( thetax )
       
       an = sqrt( ap_aq )*exp( -alpha*sn )*
     & exp( imunit*( omega*taun - caustc(ia) ) )*decay(ia)

       end if
       
c=======================================================================

c      Get the pressure:
       
      call invmat(qn,iqn,ie)

      if (ie.eq.1) then
	     
      normal(1) = rh(1) - rn(1)
      normal(2) = rh(2) - rn(2)
      normal(3) = rh(3) - rn(3)

      e1(1) = me1(ia,1)
      e1(2) = me1(ia,2)
      e1(3) = me1(ia,3)

      e2(1) = me2(ia,1)
      e2(2) = me2(ia,2)
      e2(3) = me2(ia,3)

c       Get the projection of the normal onto e1 and e2:

      call dot( e1, normal, n1 )
      call dot( e2, normal, n2 )

      n1 = abs( n1 )
      n2 = abs( n2 )

c-----------------------------------------------------------------------
      cpi = sqrt( pi )

      n1_w1 = abs( n1*iqn(1,1)/dtheta )*cpi
      n2_w2 = abs( n2*iqn(2,2)/dtheta )*cpi
      n12_w12 = abs( sqrt(n1*n2)*iqn(1,2)/dtheta )*cpi
      n21_w21 = abs( sqrt(n1*n2)*iqn(2,1)/dtheta )*cpi

c-----------------------------------------------------------------------
c     Gaussian decays:

      phin1 = exp( -(n1_w1**2.0) )
      phin2 = exp( -(n2_w2**2.0) )
      phin12 = exp( -(n12_w12**2.0) )
      phin21 = exp( -(n21_w21**2.0) )
      phin = phin1*phin2*phin12*phin21

c-----------------------------------------------------------------------
c     Ray influence:
 
      pray = phin*an
	    
      end if

c=======================================================================

c***********************************************************************
c     Back to main:
c***********************************************************************

      return
      
      end
