       subroutine r2x2( i,ibdry, r )
	  
c***********************************************************************
c      Matrix R (size 2x2) for P and Q update
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Faro, Ter Fev 14 00:14:55 WET 2017
c***********************************************************************

c***********************************************************************
c Popov and Psencik 1978:
c Computation of ray amplitudes in inhomogeneous media with curved interfaces
c (notation adapted)
c***********************************************************************

c      Include global (common) variables:

       include 'global.for'
       
       character*2 itype
       
       integer i,ibdry
       
       real*8 r(2,2)
       real*8 es(3),e1(3),e2(3)
       real*8 eta1(3),eta2(3),normal(3),taur(3)
       real*8 gradc(3),gradct(3)
       real*8 dri,dsi,dx,dy,dz
       real*8 zbdry
       real*8 ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi
       real*8 cxxi,cyyi,czzi,cxyi,cxzi,cyzi
       real*8 cit,cct,sigmait,cxit,cyit,czit,sxit,syit,szit
       real*8 cxxit,cyyit,czzit,cxyit,cxzit,cyzit
       real*8 cs,cst,c1,c1t,c2,c2t
       real*8 x1,x1t,x2,x2t
       real*8 alpha,tga,sina,cosa
       real*8 r0,r11,r12
       real*8 d11,d12,d22
       real*8 fx,fy,fn
       real*8 fxx,fyy,fxy
       real*8 f11,f12,f22
       real*8 dummy
       
c***********************************************************************
c      Let's go:
c***********************************************************************
             
       dx = x(i) - x(i-1)
       dy = y(i) - y(i-1)
       dz = z(i) - z(i-1)
      
       call csvals(x(i-1),y(i-1),z(i-1),ci,cc,sigmai,cxi,cyi,czi,
     & sxi,syi,szi,cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
      
       call csvals(x(i),y(i),z(i),cit,cct,sigmait,cxit,cyit,czit,
     & sxit,syit,szit,cxxit,cyyit,czzit,cxyit,cxzit,cyzit)
      
       dsi = s(i) - s(i-1)
       dri = sqrt( dx*dx + dy*dy )
      
       es(1) = dx/dsi
       es(2) = dy/dsi
       es(3) = dz/dsi
      
       e2(1) = -dy/dri
       e2(2) =  dx/dri
       e2(3) =   0.0
      
       gradc(1) = cxi
       gradc(2) = cyi
       gradc(3) = czi
      
       gradct(1) = cxit
       gradct(2) = cyit
       gradct(3) = czit
	     
       if (ibdry.eq.1) then 
       call fbdyi(natix,natiy,xati,yati,zati,aitype,
     & x(i),y(i),zbdry,normal,fx,fy,fxx,fyy,fxy)
       itype = aitype
       else		
       call fbdyi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,
     & x(i),y(i),zbdry,normal,fx,fy,fxx,fyy,fxy)
       itype = bitype
       normal(1) = -normal(1)
       normal(2) = -normal(2)
       normal(3) = -normal(3)            
       end if
 
       call cross(     e2,   es,   e1 )
       call cross( normal,   es, eta2 )
       call cross( eta2, normal, eta1 )

c      Calculate the coefficients of the surface 
c      (Popov and Psencik 1978):

       fn = ibdry*sqrt( fx**2 + fy**2 + 1.0 )
       
       if ((itype.eq.'3P').or.(itype.eq.'4P')) then
       f11 = fxx*eta1(1)**2 + fyy*eta1(2)**2
     &     + 2*fxy*eta1(1)*eta1(2)
       f22 = fxx*eta2(1)**2 + fyy*eta2(2)**2
     &     + 2*fxy*eta2(1)*eta2(2)
       f12 = fxx*eta1(1)*eta2(1) 
     &     + fyy*eta1(2)*eta2(2)
     &     + fxy*( eta1(1)*eta2(2) + eta1(2)*eta2(1) )
       elseif (itype.eq.'2P') then
       f11 = 2*fxy*eta1(1)*eta1(2)
       f22 = 2*fxy*eta2(1)*eta2(2)
       f12 = fxy*( eta1(1)*eta2(2) + eta1(2)*eta2(1) )
       else
       f11 = 0.0
       f12 = 0.0
       f22 = 0.0
       end if 
       
       d11 = -0.5*f11/fn
       d12 = -0.5*f12/fn
       d22 = -0.5*f22/fn
      
c      Calculate boundary reflection:

       call reflct( normal, es, taur, alpha )
      
       tga  = tan( alpha )
       sina = sin( alpha )
       cosa = cos( alpha )
      
       call dot( es , gradc , cs )
       call dot( e1 , gradc , c1 )
       call dot( e2 , gradc , c2 )
      
       call dot( es , gradct , cst )
       call dot( e1 , gradct , c1t )
       call dot( e2 , gradct , c2t )
	     
       x1  = c1 /ci
       x2  = c2 /ci
       x1t = c1t/cit
       x2t = c2t/cit
      
       r0 = cosa*( sigmai + sigmait )
      
       r11 = 2.0*sigmai*sina*( x1 + x1t ) - 2.0*d11*r0/cosa
     &  -sina*tga*( cs - cst )/cc 
      
       r12 = sigmai*sina*( x2 - x2t ) + 2.0*d12*r0

       r(1,1) = r11/cosa
       r(1,2) = r12/cosa 
       r(2,1) = sigmai*tga*( x2 - x2t ) + 2.0*d12*r0/cosa
       r(2,2) = 2.0*d22*r0

c***********************************************************************
c     Back to main:
c***********************************************************************
       
       return 
       
       end
