       subroutine readin(ctitle,nthtas,nphi,iad)

c***********************************************************************
c      READIN.FOR
c      Subroutine to READ the waveguide INput file
c      Written by Orlando Camargo Rodriguez
c      Gambelas, Sex Jan 27 13:28:30 WET 2017
c***********************************************************************

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       character(len=80) filename
       character(len=60) ctitle

       integer i,iad
       integer j,k
       
       integer nthtas,nphi
       
       real*8 dphi
       
c***********************************************************************
c      Indicator of array dimension:
c***********************************************************************

       iad = 0 
       
c***********************************************************************
c      Open the file:
c***********************************************************************

       numarg = iargc( )       

       call getarg( 1, filename )

       open(unit=inpfil,file=filename,status='old')

c***********************************************************************
c      Read the file:
c***********************************************************************

       read(inpfil,*) ctitle

c***********************************************************************
c      Source info:
c***********************************************************************

       read(inpfil,*) 
       read(inpfil,*) ds
       read(inpfil,*) xx,yx,zx
       read(inpfil,*) freqx
       read(inpfil,*) xbox(1),xbox(2)
       read(inpfil,*) ybox(1),ybox(2)
       
       if (ds.eq.0.0) then
	      
          print *,'Ray step can not be 0!'
	  stop

       end if

c-----------------------------------------------------------------------
       read(inpfil,*) nthtas
c-----------------------------------------------------------------------
	  if (abs(nthtas).gt.ntmax) then
	      
	      close(inpfil)
	      print *, 'inpfil: |NTHETAS| > ',ntmax,'!'
	      print *, 'aborting calculations...'
	      stop

c-----------------------------------------------------------------------
	  else if (nthtas.eq.1) then
	      read(inpfil,*) thetas(1)
              dtheta = 1.0

c-----------------------------------------------------------------------
	  else if (nthtas.eq.2) then
	      read(inpfil,*) thetas(1),thetas(2)
              dtheta = thetas(2) - thetas(1)

c-----------------------------------------------------------------------
	  else if (nthtas.lt.0) then
	      read(inpfil,*) (thetas(i),i=1,abs(nthtas))
              dtheta = 1.0

c-----------------------------------------------------------------------
	  else
	      read(inpfil,*) thetas(1),thetas(nthtas)
	      dtheta = (thetas(nthtas)-thetas(1))/(nthtas-1)
	      do i = 2,nthtas-1
	      thetas(i) = thetas(1) + dtheta*(i-1)
	      end do
	  end if
          
	  nthtas = abs( nthtas )

c-----------------------------------------------------------------------
       read(inpfil,*) nphi
c-----------------------------------------------------------------------
	  if (abs(nphi).gt.npmax) then

	      close(inpfil)
	      print *, 'inpfil: |NPHI| > ',npmax,'!'
	      print *, 'aborting calculations...'
	      stop
c-----------------------------------------------------------------------
	  else if (nphi.eq.1) then
	      read(inpfil,*) phi(1)

c-----------------------------------------------------------------------
	  else if (nphi.eq.2) then
	      read(inpfil,*) phi(1),phi(2)

c-----------------------------------------------------------------------
	  else if (nphi.lt.0) then
	      read(inpfil,*) (phi(i),i=1,abs(nphi))

c-----------------------------------------------------------------------

	  else 
	      read(inpfil,*) phi(1),phi(nphi)
	      dphi = (phi(nphi)-phi(1))/(nphi-1)
	      do i = 2,nphi-1
	      phi(i) = phi(1) + dphi*(i-1)
	      end do
	  end if
	  
	  nphi = abs( nphi )

c***********************************************************************
c     Read altimetry info:
c***********************************************************************

       read(inpfil,*)
       read(inpfil,*)  atype
       read(inpfil,*) aptype
       read(inpfil,*) aitype
       read(inpfil,*) atiu
       
c-----------------------------------------------------------------------
       if ((aitype.ne.'2P').and.(aitype.ne.'3P').and.(aitype.ne.'4P').
     & and.(aitype.ne.'FL')) then
      
          print *, 'Surface interpolation:'
          print *, 'unknown interpolation type,'
          print *, 'aborting calculations...'
          stop
      
       end if
c-----------------------------------------------------------------------
       if ((atiu.ne.'F').and.(atiu.ne.'M').and.(atiu.ne.'N').
     & and.(atiu.ne.'Q').and.(atiu.ne.'W')) then
      
          print *, 'Surface attenuation: unknown units,'
          print *, 'aborting calculations...'
          stop
      
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) natix

c-----------------------------------------------------------------------
       if (natix.gt.nbdry) then

           print *, 'altimetry points along X > ',nbdry,'!'
	   print *, 'aborting calculations...'
           stop
           
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) (xati(i),i=1,natix)

c-----------------------------------------------------------------------

       read(inpfil,*) natiy

c-----------------------------------------------------------------------
       if (natiy.gt.nbdry) then

           print *, 'altimetry points  along Y > ',nbdry,'!'
	   print *, 'aborting calculations...'
           stop
           
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) (yati(i),i=1,natiy)

c-----------------------------------------------------------------------

       do j = 1,natiy
       read(inpfil,*) (zati(j,i),i=1,natix)
       end do
       
       if (aptype.eq.'H') then
	  
       read(inpfil,*) cpati,csati,rhoati,apati,asati
       
       elseif (aptype.eq.'N') then
	        
       do j = 1,natiy
       read(inpfil,*) (cpa(j,i),i=1,natix)
       end do
       
       do j = 1,natiy
       read(inpfil,*) (csa(j,i),i=1,natix)
       end do
       
       do j = 1,natiy
       read(inpfil,*) (rhoa(j,i),i=1,natix)
       end do
       
       do j = 1,natiy
       read(inpfil,*) (apa(j,i),i=1,natix)
       end do
       
       do j = 1,natiy
       read(inpfil,*) (asa(j,i),i=1,natix)
       end do
       
       else
       
	   print *, 'inpfil: Unknown surface properties,'
           print *, 'aborting calculations...'
	   stop

       end if

c***********************************************************************
c     Read sound speed info:
c***********************************************************************

       read(inpfil,*)
       read(inpfil,*) cdist
       
       if ((cdist(1:1).ne.'C')) then

             read(inpfil,*)	     
             read(inpfil,*) z0(1), c0(1)
	     read(inpfil,*) z0(2), c0(2)
       
       elseif ((cdist.eq.'CX00')) then
       
             read(inpfil,*) nx0
	     read(inpfil,*) (x0(i),i=1,nx0)
             read(inpfil,*) (c0(i),i=1,nx0)

       elseif ((cdist.eq.'C0Y0')) then
       
             read(inpfil,*) ny0
	     read(inpfil,*) (y0(i),i=1,ny0)
             read(inpfil,*) (c0(i),i=1,ny0)
	     
       elseif ((cdist.eq.'C00Z')) then

             read(inpfil,*) nz0
	     read(inpfil,*) (z0(i),i=1,nz0)
             read(inpfil,*) (c0(i),i=1,nz0)

       elseif ((cdist.eq.'CXY0')) then
       
             read(inpfil,*) nx0,ny0
	     read(inpfil,*) (x0(i),i=1,nx0)
             read(inpfil,*) (y0(i),i=1,ny0)
	     do j = 1,ny0
	     read(inpfil,*) (c02d(j,i),i=1,nx0)
	     end do
	     
       elseif ((cdist.eq.'CX0Z')) then
       
             read(inpfil,*) nx0,nz0
	     read(inpfil,*) (x0(i),i=1,nx0)
             read(inpfil,*) (z0(i),i=1,nz0)
	     do j = 1,ny0
	     read(inpfil,*) (c02d(j,i),i=1,nx0)
	     end do
	     
       elseif ((cdist.eq.'C0YZ')) then
       
             read(inpfil,*) ny0,nz0
	     read(inpfil,*) (y0(i),i=1,ny0)
             read(inpfil,*) (z0(i),i=1,nz0)
	     do j = 1,ny0
	     read(inpfil,*) (c02d(j,i),i=1,ny0)
	     end do

       elseif ((cdist.eq.'CXYZ')) then
       
             read(inpfil,*) nx0,ny0,nz0
	     read(inpfil,*) (x0(i),i=1,nx0)
	     read(inpfil,*) (y0(i),i=1,ny0)
             read(inpfil,*) (z0(i),i=1,nz0)
	     do k = 1,nz0
	     do j = 1,ny0
	     read(inpfil,*) (c03d(k,j,i),i=1,nx0)
	     end do
	     end do

       else
       
           print *, 'Sound speed type unknown,'
	   print *, 'aborting calculations...'
           stop
       
       end if

c***********************************************************************
c     Read object info:
c***********************************************************************
c      Incomplete...
       read(inpfil,*) 
       read(inpfil,*) nobj

       if (nobj.gt.nmaxo) then
        print *, 'inpfil: # of objects >',nmaxo,'!'
        print *, 'aborting calculations...'
        stop 
       end if

c***********************************************************************
c     Read bathymetry info:
c***********************************************************************

       read(inpfil,*)
       read(inpfil,*)  btype
       read(inpfil,*) bptype
       read(inpfil,*) bitype
       read(inpfil,*) btyu

c-----------------------------------------------------------------------
       if ((bitype.ne.'2P').and.(bitype.ne.'3P').and.(bitype.ne.'4P').
     & and.(bitype.ne.'FL')) then
      
          print *, 'Bottom interpolation:'
          print *, 'unknown interpolation type,'
          print *, 'aborting calculations...'
          stop
      
       end if
c-----------------------------------------------------------------------
       if ((btyu.ne.'F').and.(btyu.ne.'M').and.(btyu.ne.'N').
     & and.(btyu.ne.'Q').and.(btyu.ne.'W')) then
      
          print *, 'Bottom attenuation: unknown units,'
          print *, 'aborting calculations...'
          stop
      
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) nbtyx

c-----------------------------------------------------------------------
       if (nbtyx.gt.nbdry) then

           print *, 'bathymetry points along X > ',nbdry,'!'
	   print *, 'aborting calculations...'
           stop
           
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) (xbty(i),i=1,nbtyx)

c-----------------------------------------------------------------------

       read(inpfil,*) nbtyy

c-----------------------------------------------------------------------
       if (nbtyy.gt.nbdry) then

           print *, 'bathymetry points  along Y > ',nbdry,'!'
	   print *, 'aborting calculations...'
           stop
           
       end if
c-----------------------------------------------------------------------

       read(inpfil,*) (ybty(i),i=1,nbtyy)

c-----------------------------------------------------------------------

       do j = 1,nbtyy
       read(inpfil,*) (zbty(j,i),i=1,nbtyx)
       end do
       
       if (bptype.eq.'H') then
	  
       read(inpfil,*) cpbty,csbty,rhobty,apbty,asbty

       elseif (bptype.eq.'N') then

       do j = 1,nbtyy
       read(inpfil,*) (cpb(j,i),i=1,nbtyx)
       end do
       
       do j = 1,nbtyy
       read(inpfil,*) (csb(j,i),i=1,nbtyx)
       end do
       
       do j = 1,nbtyy
       read(inpfil,*) (rhob(j,i),i=1,nbtyx)
       end do
       
       do j = 1,nbtyy
       read(inpfil,*) (apb(j,i),i=1,nbtyx)
       end do
       
       do j = 1,nbtyy
       read(inpfil,*) (asb(j,i),i=1,nbtyx)
       end do
       
       else
       
	   print *, 'inpfil: Unknown bottom properties,'
           print *, 'aborting calculations...'
	   stop

       end if

c***********************************************************************
c     Read array info:
c***********************************************************************

       read(inpfil,*)
       read(inpfil,*) nxa,nya,nza

       na = min(nxa,nya,nza)

       if (na.gt.1) then

	  print *, 'inpfil: min(nxa,nya,nza)> 1'
	  print *, '(3D field calculations not supported),'
	  print *, 'aborting calculations...'
	  stop

       end if
       
       na = max(nxa,nya,nza)

       if (na.gt.nc0) then

	  print *, 'inpfil: max(nxa,nya,nza)>',nc0
	  print *, 'aborting calculations...'
	  stop

       end if

       if ((nxa*nya*nza).eq.0) then
	  
 	  do i = 1,na
 
 	     read(inpfil,*) xarray(i),yarray(i),zarray(i)

 	  end do

       else
 
 	  read(inpfil,*) (xarray(i),i=1,nxa)
          read(inpfil,*) (yarray(i),i=1,nya)
          read(inpfil,*) (zarray(i),i=1,nza)

       end if

       if ((nxa*nya*nza).eq.0) then
       iad = 1
       elseif (min(nxa,nya,nza).eq.1) then
       iad = 2
       else 
       iad = 3
       end if

c***********************************************************************
c     Read output calculations info:
c***********************************************************************

       read(inpfil,*)
       read(inpfil,*) catype
       read(inpfil,*) miss

c***********************************************************************
c      Close the file:
c***********************************************************************

       close(inpfil)

c***********************************************************************
c     Source check:
c***********************************************************************

       call sbdyi(natix,natiy,xati,yati,zati,aitype,xx,yx,zatix)
       call sbdyi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,xx,yx,zbtyx)
     
       if (zatix.gt.zx) then
        print *, 'inpfil: source is located above the surface!'
        print *, 'aborting calculations...'
        stop
       end if
       
       if (zbtyx.lt.zx) then
        print *, 'inpfil: source is located below the bottom!'
        print *, 'aborting calculations...'
        stop
       end if
       
c***********************************************************************
c     Bathymetry/altimetry check:
c***********************************************************************

       if (xati(1).gt.xbox(1)) then
        print *, 'inpfil: minimal altimetry X >'
        print *, 'inpfil: minimal box       X!'
        print *, 'aborting calculations...'
        stop
       end if 
       
       if (xati(natix).lt.xbox(2)) then
        print *, 'inpfil: maximal altimetry X <'
        print *, 'inpfil: maximal box       X!'
        print *, 'aborting calculations...'
        stop
       end if
       
       if (yati(1).gt.ybox(1)) then
        print *, 'inpfil: minimal altimetry Y >'
        print *, 'inpfil: minimal box       Y!'
        print *, 'aborting calculations...'
        stop
       end if 
       
       if (yati(natiy).lt.ybox(2)) then
        print *, 'inpfil: maximal altimetry Y <'
        print *, 'inpfil: maximal box       Y!'
        print *, 'aborting calculations...'
        stop
       end if 
     
       if (xbty(1).gt.xbox(1)) then
        print *, 'inpfil: minimal bathymetry X >'
        print *, 'inpfil: minimal box        X!'
        print *, 'aborting calculations...'
        stop
       end if
       
       if (xbty(nbtyx).lt.xbox(2)) then
        print *, 'inpfil: maximal bathymetry X <'
        print *, 'inpfil: maximal box        X!'
        print *, 'aborting calculations...'
        stop
       end if

       if (ybty(1).gt.ybox(1)) then
        print *, 'inpfil: minimal bathymetry Y >'
        print *, 'inpfil: minimal box        Y!'
        print *, 'aborting calculations...'
        stop
       end if
       
       if (ybty(nbtyy).lt.ybox(2)) then
        print *, 'inpfil: maximal bathymetry Y <'
        print *, 'inpfil: maximal box        Y!'
        print *, 'aborting calculations...'
        stop
       end if

c***********************************************************************
c     Back to main:
c***********************************************************************

      return
      
      end
