       subroutine rkf45(dsi,yold,fold,ynew,fnew,ds4,ds5)
       
c***********************************************************************
c      Runge-Kutta-Fehlberg integrator
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Gambelas, Sex Fev 24 18:09:03 WET 2017
c****&******************************************************************

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       integer j
 
       real*8 yold(6),ynew(6),yrk4(6),yrk5(6),yk(6)
       real*8 fold(6),fnew(6)
       
       real*8 k1(6),k2(6),k3(6),k4(6),k5(6),k6(6)

       real*8 es(3)
       
       real*8 a1,a3,a4,a5
       real*8 b1,b3,b4,b5,b6
       real*8 xi,yi,zi
       real*8 ci,cc
       real*8 cxi,cyi,czi
       real*8 cxxi,cyyi,czzi,cxyi,cxzi,cyzi
       real*8 sigmai,sigmax,sigmay,sigmaz
       real*8 dsi,ds4,ds5
       real*8 sxi,syi,szi
       
c***********************************************************************
       
c***********************************************************************

       a1 = 25.0/216.0
       a3 = 1408.0/2565.0
       a4 = 2197.0/4101.0
       a5 = -1.0/5.0
       
       b1 = 16.0/135.0
       b3 = 6656.0/12825.0
       b4 = 28561.0/56430.0
       b5 = -9.0/50.0
       b6 = 2.0/55.0

c***********************************************************************

       do j = 1,6

          k1(j) = fold(j)
          yk(j) = yold(j) + 0.25*dsi*k1(j)
 
       end do

           xi = yk(1)
           yi = yk(2)
	   zi = yk(3)
       sigmax = yk(4)
       sigmay = yk(5)
       sigmaz = yk(6)
       
       sigmai = sqrt( sigmax**2 + sigmay**2+ sigmaz**2 )

       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
       
       k2(1) = es(1)
       k2(2) = es(2)
       k2(3) = es(3)
       k2(4) = sxi
       k2(5) = syi
       k2(6) = szi
        
       do j = 1,6
	
	  yk(j) = yold(j) + dsi*(3.0/32.0*k1(j)+9.0/32.0*k2(j))
	
       end do

           xi = yk(1)
           yi = yk(2)
	   zi = yk(3)
       sigmax = yk(4)
       sigmay = yk(5)
       sigmaz = yk(6)
       
       sigmai = sqrt( sigmax**2 + sigmay**2+ sigmaz**2 )

       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
       
       k3(1) = es(1)
       k3(2) = es(2)
       k3(3) = es(3)
       k3(4) = sxi
       k3(5) = syi
       k3(6) = szi

       do j = 1,6
	
	  yk(j) = yold(j) + dsi*(1932.0/2197.0*k1(j)
     &      -7200.0/2197.0*k2(j)+7296.0/2197.0*k3(j))
	
       end do
        
           xi = yk(1)
           yi = yk(2)
	   zi = yk(3)
       sigmax = yk(4)
       sigmay = yk(5)
       sigmaz = yk(6)
       
       sigmai = sqrt( sigmax**2 + sigmay**2+ sigmaz**2 )

       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
       
       k4(1) = es(1)
       k4(2) = es(2)
       k4(3) = es(3)
       k4(4) = sxi
       k4(5) = syi
       k4(6) = szi
       
       do j = 1,6
	
	  yk(j) = yold(j) + dsi*(439.0/216.0*k1(j)-8.0*k2(j)
     &        +3680.0/513.0*k3(j)-845.0/4104*k4(j))
	
       end do
        
           xi = yk(1)
           yi = yk(2)
	   zi = yk(3)
       sigmax = yk(4)
       sigmay = yk(5)
       sigmaz = yk(6)
       
       sigmai = sqrt( sigmax**2 + sigmay**2+ sigmaz**2 )

       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
       
       k5(1) = es(1)
       k5(2) = es(2)
       k5(3) = es(3)
       k5(4) = sxi
       k5(5) = syi
       k5(6) = szi

       do j = 1,6
	
	  yk(j) = yold(j) + dsi*(2.0*k2(j)-8.0/27.0*k1(j)
     &   +3544.0/2565.0*k3(j)+1859.0/4104*k4(j)-11.0/40.0*k5(j))
	
       end do

           xi = yk(1)
           yi = yk(2)
	   zi = yk(3)
       sigmax = yk(4)
       sigmay = yk(5)
       sigmaz = yk(6)
       
       sigmai = sqrt( sigmax**2 + sigmay**2+ sigmaz**2 )

       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
       
       k6(1) = es(1)
       k6(2) = es(2)
       k6(3) = es(3)
       k6(4) = sxi
       k6(5) = syi
       k6(6) = szi

       do j = 1,6
	
	  yrk4(j)=yold(j)+dsi*(a1*k1(j)+a3*k3(j)+a4*k4(j)+a5*k5(j))
	  yrk5(j)=yold(j)+dsi*(b1*k1(j)+b3*k3(j)+b4*k4(j)+b5*k5(j)
     &           +b6*k6(j))
	  
c	  ynew(j) = yrk4(j)
	  ynew(j) = yrk5(j)
	  
       end do

           xi = ynew(1)
           yi = ynew(2)
	   zi = ynew(3)
       sigmax = ynew(4)
       sigmay = ynew(5)
       sigmaz = ynew(6)
              
       dx = yrk4(1) - yold(1)
       dy = yrk4(2) - yold(2)
       dz = yrk4(3) - yold(3)
           
       ds4 = sqrt(dx*dx+dy*dy+dz*dz)
             
       dx = yrk5(1) - yold(1)
       dy = yrk5(2) - yold(2)
       dz = yrk5(3) - yold(3)
           
       ds5 = sqrt(dx*dx+dy*dy+dz*dz)
           
       sigmai = sqrt( sigmax**2 + sigmay**2 + sigmaz**2 )
       
       es(1) = sigmax/sigmai
       es(2) = sigmay/sigmai
       es(3) = sigmaz/sigmai

       call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

       fnew(1) = es(1)
       fnew(2) = es(2)
       fnew(3) = es(3)
       fnew(4) = sxi
       fnew(5) = syi
       fnew(6) = szi

c*********************************************************************** 
c      Back to tracer...
c*********************************************************************** 
       
       return 
       
       end 

