       subroutine sdyneq(thetax,phix,imax,irefl)

c**********************************************************************
c      Solve DYNamic EQuations
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Gambelas, Sex Fev 24 16:49:29 WET 2017
c      Leap-frog integration
c****&******************************************************************

c      Include global (common) variables:

       include 'global.for'
       
c      Define local variables:
       
       integer irefl(np)
       
       integer i,imax,j,k
       integer ibdry
       
       real*8 c(2,2),cq(2,2),q2x2(2,2),r(2,2)
       
       real*8 thetax,phix,ci,cc,c11,c22,c12
       real*8 cxi,cyi,czi
       real*8 cxxi,cyyi,czzi,cxyi,cxzi,cyzi
       real*8 sigmai,sxi,syi,szi
       real*8 zatii,zbtyi
       real*8 dx,dy,dz,dsi
       real*8 alpha
       real*8 ap_aq,prod

c***********************************************************************

       lambda = cx/freqx

c***********************************************************************
c i1 always lines on the vertical  plane containing es;
c i2 always lies on the horizontal plane XY;
c e1 and e2 are projected onto i1 and i2; 
c the angle of projection depends on the local azimuth and elevation of es.
c***********************************************************************

c***********************************************************************
c Popov and Psencik 1978:
c Computation of ray amplitudes in inhomogeneous media with curved interfaces
c (notation adapted)
c***********************************************************************

c**********************************************************************
c      YES, detQ IS EXPECTED TO CHANGE SIGN, THIS IS WHY THE RAY AMPLITUDE 
C      WON'T ALWAYS DECREASE STEADILY OVER RANGE!!!!!!!!!!!!!!!!!!!!!!!
c****&******************************************************************

c      Get Thorpe attenuation in dB/m: 

       call thorpe(freqx,alpha)

c-----------------------------------------------------------------------
c      Define initial conditions:
       
       ibdry = 0
       
       detq(1) = 1.0
       
       caustc(1) = 0.0

       p(1,1,1) = 1.0/cx
       p(1,1,2) = 0.0
       p(1,2,1) = 0.0
       p(1,2,2) = 1.0/cx

       q(1,1,1) = 0.0
       q(1,1,2) = 0.0
       q(1,2,1) = 0.0
       q(1,2,2) = 0.0
       
       dsi = s(2) - s(1)
       
       me1(1,1) = -sin( rayel(1) )*cos( rayaz(1) )
       me1(1,2) = -sin( rayel(1) )*sin( rayaz(1) )
       me1(1,3) =  cos( rayel(1) )
        
       me2(1,1) = -sin( rayaz(1) )
       me2(1,2) =  cos( rayaz(1) )
       me2(1,3) =  0.0
       
       ibdry = irefl(2)
       
       if  (ibdry.eq.0) then     
c-----------------------------------------------------------------------
c Get matrix C:

       call csvals(x(1),y(1),z(1),ci,cc,sigmai,cxi,cyi,czi,
     & sxi,syi,szi,cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

       c11 = ( me1(1,1)**2  )*cxxi
     &     + ( me1(1,2)**2  )*cyyi
     &     + ( me1(1,3)**2  )*czzi
     &     + 2*me1(1,1)*me1(1,2)*cxyi
     &     + 2*me1(1,1)*me1(1,3)*cxzi
     &	   + 2*me1(1,2)*me1(1,3)*cyzi
     
       c12 = ( me1(1,1)*me2(1,1) )*cxxi
     &     + ( me1(1,2)*me2(1,2) )*cyyi
     &	   + ( me1(1,3)*me2(1,3) )*czzi
     &     + ( me1(1,1)*me2(1,2) + me1(1,2)*me2(1,1) )*cxyi
     &     + ( me1(1,1)*me2(1,3) + me1(1,3)*me2(1,1) )*cxzi
     &     + ( me1(1,2)*me2(1,3) + me1(1,3)*me2(1,2) )*cyzi
          
       c22 = ( me2(1,1)**2  )*cxxi
     &     + ( me2(1,2)**2  )*cyyi
     &     + ( me2(1,3)**2  )*czzi
     &     + 2*me2(1,1)*me2(1,2)*cxyi
     &     + 2*me2(1,1)*me2(1,3)*cxzi
     &     + 2*me2(1,2)*me2(1,3)*cyzi

       c(1,1) = c11
       c(1,2) = c12
       c(2,1) = c12
       c(2,2) = c22

c-----------------------------------------------------------------------
c Get detQ and matrix CQ:

       q2x2(1,1) = q(1,1,1)
       q2x2(1,2) = q(1,1,2)
       q2x2(2,1) = q(1,2,1)
       q2x2(2,2) = q(1,2,2)
       
       call det2x2(q2x2,detq(1))
	
       call matpro(c,q2x2,cq)

c-----------------------------------------------------------------------
       
       do j = 1,2
       
          do k = 1,2
	  
	  q(2,j,k) = q(1,j,k) + dsi* p(1,j,k)*ci
	  p(2,j,k) = p(1,j,k) - dsi*  cq(j,k)/cc
	  
	  end do
       
       end do

      else
       
       q(2,1,1) = -q(1,1,1)
       q(2,1,2) = -q(1,1,2)
       q(2,2,1) =  q(1,2,1)
       q(2,2,2) =  q(1,2,2)
       
       call r2x2(2,ibdry,r)

       p(2,1,1) = -p(1,1,1) - q(1,1,1)*r(1,1) - q(1,2,1)*r(1,2)
       p(2,1,2) = -p(1,1,2) - q(1,1,2)*r(1,1) - q(1,2,2)*r(1,2)
       p(2,2,1) =  p(1,2,1) + q(1,1,1)*r(2,1) - q(1,2,1)*r(2,2)
       p(2,2,2) =  p(1,2,2) + q(1,1,2)*r(2,1) - q(1,2,2)*r(2,2)
      
      end if
c-----------------------------------------------------------------------
c      Solve the Dynamic Equations:

       do i = 2,imax-1

       ibdry = irefl(i+1)
       
       dsi = s(i+1) - s(i-1)

c-----------------------------------------------------------------------
c     Update matrices with e1 and e2:
 
       me1(i,1) = -sin( rayel(i) )*cos( rayaz(i) )
       me1(i,2) = -sin( rayel(i) )*sin( rayaz(i) )
       me1(i,3) =  cos( rayel(i) )
        
       me2(i,1) = -sin( rayaz(i) )
       me2(i,2) =  cos( rayaz(i) )
       me2(i,3) =  0.0
      
c-----------------------------------------------------------------------
c Get matrix C:

       call csvals(x(i),y(i),z(i),ci,cc,sigmai,cxi,cyi,czi,
     & sxi,syi,szi,cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

       c11 = ( me1(i,1)**2  )*cxxi
     &     + ( me1(i,2)**2  )*cyyi
     &     + ( me1(i,3)**2  )*czzi
     &     + 2*me1(i,1)*me1(i,2)*cxyi
     &     + 2*me1(i,1)*me1(i,3)*cxzi
     &	   + 2*me1(i,2)*me1(i,3)*cyzi
     
       c12 = ( me1(i,1)*me2(i,1) )*cxxi
     &     + ( me1(i,2)*me2(i,2) )*cyyi
     &	   + ( me1(i,3)*me2(i,3) )*czzi
     &     + ( me1(i,1)*me2(i,2) + me1(i,2)*me2(i,1) )*cxyi
     &     + ( me1(i,1)*me2(i,3) + me1(i,3)*me2(i,1) )*cxzi
     &     + ( me1(i,2)*me2(i,3) + me1(i,3)*me2(i,2) )*cyzi
          
       c22 = ( me2(i,1)**2  )*cxxi
     &     + ( me2(i,2)**2  )*cyyi
     &     + ( me2(i,3)**2  )*czzi
     &     + 2*me2(i,1)*me2(i,2)*cxyi
     &     + 2*me2(i,1)*me2(i,3)*cxzi
     &     + 2*me2(i,2)*me2(i,3)*cyzi

       c(1,1) = c11
       c(1,2) = c12
       c(2,1) = c12
       c(2,2) = c22

c-----------------------------------------------------------------------
c Get detQ and matrix CQ:

       q2x2(1,1) = q(i,1,1)
       q2x2(1,2) = q(i,1,2)
       q2x2(2,1) = q(i,2,1)
       q2x2(2,2) = q(i,2,2)
       
       call det2x2(q2x2,detq(i))
	
       call matpro(c,q2x2,cq)

c-----------------------------------------------------------------------
c Update P and Q for either refraction or reflection:
       
       if (ibdry.ne.0) then
       
       q(i+1,1,1) = -q(i,1,1)
       q(i+1,1,2) = -q(i,1,2)
       q(i+1,2,1) =  q(i,2,1)
       q(i+1,2,2) =  q(i,2,2)
       
       call r2x2(i+1,ibdry,r)

       p(i+1,1,1) = -p(i,1,1) - q(i,1,1)*r(1,1) - q(i,2,1)*r(1,2)
       p(i+1,1,2) = -p(i,1,2) - q(i,1,2)*r(1,1) - q(i,2,2)*r(1,2)
       p(i+1,2,1) =  p(i,2,1) + q(i,1,1)*r(2,1) - q(i,2,1)*r(2,2)
       p(i+1,2,2) =  p(i,2,2) + q(i,1,2)*r(2,1) - q(i,2,2)*r(2,2)
       
       elseif ((ibdry.eq.0).and.(irefl(i).ne.0)) then
       
       do j = 1,2
       
          do k = 1,2
	  
	  q(i+1,j,k) = q(i,j,k) + (s(i+1)-s(i))* p(i,j,k)*ci
	  p(i+1,j,k) = p(i,j,k) - (s(i+1)-s(i))*cq(j,k)/cc
	  
	  end do
       
       end do
       
       else
       
       do j = 1,2
       
          do k = 1,2
	  
	  q(i+1,j,k) = q(i-1,j,k) + dsi* p(i,j,k)*ci
	  p(i+1,j,k) = p(i-1,j,k) - dsi*  cq(j,k)/cc
	  
	  end do
       
       end do

      end if
       
c-----------------------------------------------------------------------

       end do
	  
c***********************************************************************
c      Acoustic pressure along the ray:
c***********************************************************************
       
       detq(imax) = q(imax,1,1)*q(imax,2,2)-q(imax,1,2)*q(imax,2,1)
       
       do i = 2,imax

	   ibdry = irefl(i)
	   
	   prod = detq(i-1)*detq(i)
       
           if ((prod.le.0.0).and.(detq(i-1).ne.0.0)
     &     .and.(ibdry.eq.0.)) then
           caustc(i) = caustc(i-1) + 0.5*pi
           else
           caustc(i) = caustc(i-1)
           end if 
        
	   ap_aq = cos(thetax)/abs( detq(i) )*cray(i)/cx
	   
	   amp(i) = sqrt( ap_aq )*exp( -alpha*s(i) )*decay(i)

       end do

c    Let's fill missing quantities:

      amp(1) = amp(2)
      
      me1(imax,1) = me1(imax-1,1)
      me1(imax,2) = me1(imax-1,2)
      me1(imax,3) = me1(imax-1,3)
 
      me2(imax,1) = me2(imax-1,1)
      me2(imax,2) = me2(imax-1,2)
      me2(imax,3) = me2(imax-1,3)
      
c***********************************************************************
c     Back to main:
c***********************************************************************

       return

       end
