       subroutine seikeq(thetax,phix,imax,irefl)

c**********************************************************************
c      Solve EIKonal EQuations
c      Written by Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Faro, Sex Jul 12 10:28:21 WEST 2019
c****&******************************************************************

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:

       integer irefl(np)

       integer i,j,k,imax,iout
       integer jrefl
       integer nrk

       real*8 yold(6),ynew(6),fold(6),fnew(6)
       real*8 es(3),eso(3)
       real*8 normal(3),taur(3),v(3)
       real*8 la(3),lb(3),li(3)
       real*8 p0(3),p1(3),p2(3)
       
       real*8 thetax,phix,theta
       real*8 ci,cc,cxi,cyi,czi,sxi,syi,szi
       real*8 cxxi,cyyi,czzi,cxyi,cxzi,cyzi
       real*8 sigmax,sigmay,sigmaz,sigmai
       real*8 dx,dy,dz,dsi,dri,ds4,ds5,dsl,dt
       real*8 zatii,zbtyi
       real*8 rho1,rho2,cp2,cs2,ap2,as2
       real*8 turn
       real*8 erros
       real*8 d1,d2,d3,d4,d5

       complex*16 refl,refldc

c***********************************************************************

       rho1 = 1.0

c***********************************************************************

c      Define initial conditions:
       
       i      = 1
       ikill  = 0
       iout   = 0
       iretrn = 0
       jrefl  = 0
       nrk    = 0
       
        srefl = 0
	brefl = 0
	orefl = 0
        nrefl = 0
        
       irefl(1) = 0
       
       rold = 0.0
       turn = 0.0
       
       call csvals(xx,yx,zx,cx,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     & cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

       sigmax = sigmai*cos( thetax )*cos( phix )
       sigmay = sigmai*cos( thetax )*sin( phix )
       sigmaz = sigmai*sin( thetax )
       
       yold(1) = xx
       yold(2) = yx
       yold(3) = zx 
       yold(4) = sigmax
       yold(5) = sigmay
       yold(6) = sigmaz

       fold(1) = sigmax/sigmai
       fold(2) = sigmay/sigmai
       fold(3) = sigmaz/sigmai
       fold(4) = 0
       fold(5) = 0
       fold(6) = 0
       
       mes(1,1) = fold(1)
       mes(1,2) = fold(2)
       mes(1,3) = fold(3)

       cray(1) = cx
        tau(1) = 0.0
          s(1) = 0.0
       
       x(1) = xx
       y(1) = yx
       z(1) = zx

       xmin = x(1)
       xmax = x(1)
       ymin = y(1)
       ymax = y(1)
       
       refldc = (1.0,0.0)

       decay(1) = refldc

       rayel(1) = thetax
       rayaz(1) =   phix
       
c-----------------------------------------------------------------------
c      Trace the ray:

1000   if (( x(i).lt.xbox(2) ).and.( x(i).gt.xbox(1) ).and.
     &     ( y(i).lt.ybox(2) ).and.( y(i).gt.ybox(1) ).and.
     &     ( ikill.eq.0 )) then
 
          dsi = ds

c         Runge-Kutta integration:

2000       call rkf45(dsi,yold,fold,ynew,fnew,ds4,ds5)

           eso(1) = fold(1)
           eso(2) = fold(2)
           eso(3) = fold(3)
	   
            es(1) = fnew(1)
            es(2) = fnew(2)
            es(3) = fnew(3)
	    
           erros = 1.0-abs(eso(1)*es(1)+eso(2)*es(2)+eso(3)*es(3))

c	   if (erros.gt.2.0e-6) then
c	   if (erros.gt.2.0e-8) then
c	   if (erros.gt.5.0e-9) then	   
c          THIS IS TOO DEMANDING:
	   if (erros.gt.3.0e-9) then
c	   if (erros.gt.2.0e-9) then
c	   if (erros.gt.1.0e-9) then
c	   if (erros.gt.2.0e-10) then 
	   
	   nrk = nrk + 1 
	   
	     if (nrk.gt.100) then
	     print *, 'Runge-Kutta integration: failure in step'
	     print *, 'convergence, aborting calculations...'
	     stop 
	     end if
           
	   dsi = 0.5*dsi
           goto 2000
           
	   end if

	  es(1) = fnew(1)
          es(2) = fnew(2)
	  es(3) = fnew(3)

c         Test for boundary intersections:
          
          xi = ynew(1)
          yi = ynew(2)
	  zi = ynew(3)

          if ((xi.ge.xbox(1)).and.(xi.le.xbox(2)).and.
     &        (yi.ge.ybox(1)).and.(yi.le.ybox(2))) then
	  call sbdyi(natix,natiy,xati,yati,zati,aitype,xi,yi,zatii)
          call sbdyi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,xi,yi,zbtyi)
          else 
	  ikill = 1
	  end if

c[====================================================================]
c         Check if the ray is still between the boundaries; if not 
c         find the intersection point and calculate the reflection:
          
          if (((zi.lt.zatii).or.(zi.gt.zbtyi)).and.(ikill.eq.0)) then
           
          la(1) = yold(1)
          la(2) = yold(2)
	  la(3) = yold(3)
          
          lb(1) = ynew(1)
          lb(2) = ynew(2)
	  lb(3) = ynew(3)

c======================================================================
c         Ray above surface?
           
          if (zi.lt.zatii) then
	  
	     jrefl = 1
             srefl = srefl + 1
             nrefl = nrefl + 1
	     
 	     if (nrefl.gt.nreflx) then
 	     print *, 'Reflection points: nrefl > ',nreflx
 	     print *, 'aborting calculations...'
 	     stop
 	     end if

             call raybi(natix,natiy,xati,yati,zati,aitype,la,lb,li)

             xi = li(1)
             yi = li(2)
	     zi = li(3)

             krefl(nrefl) = i+1
             
	     call fbdyi(natix,natiy,xati,yati,zati,aitype,
     &       xi,yi,zatii,normal,d1,d2,d3,d4,d5)

c            Calculate surface reflection:

             call reflct( normal, es, taur, theta )
	     
c            Check if the ray is digging in beyond the surface:

             call sbdyi(natix,natiy,xati,yati,zati,aitype,
     &       xi+ds*taur(1),yi+ds*taur(2),zatii)
	     
	     if ((zi+ds*taur(3)).lt.zatii) then
	     ikill = 1
	     end if
	     
c            Surface reflection => get the reflection coefficient
c            (kill the ray is the surface is an absorver):

             if (atype.eq.'A') then
             refl = (0.0,0.0)
             ikill = 1
             elseif (atype.eq.'R') then
             refl = (1.0,0.0)
             elseif (atype.eq.'V') then
             refl = (-1.0,0.0)
             elseif (atype.eq.'E') then
               if (aptype.eq.'H') then
		  lambda = cpati/freqx
                  call cnvnts(apati,lambda,freqx,atiu,adBoW)
                  ap2  = adBoW
		  lambda = csati/freqx
                  call cnvnts(asati,lambda,freqx,atiu,adBoW)
                  as2  = adBoW
                  call bdryr(rho1,rhoati,ci,cpati,csati,ap2,as2,
     &            theta,refl)
               elseif (aptype.eq.'N') then
	       call sbdyi(natix,natiy,xati,yati,rhoa,aitype,
     &         xi,yi,rho2)
	       call sbdyi(natix,natiy,xati,yati,cpa,aitype,
     &         xi,yi,cp2)
	       call sbdyi(natix,natiy,xati,yati,csa,aitype,
     &         xi,yi,cs2)
	       call sbdyi(natix,natiy,xati,yati,apa,aitype,
     &         xi,yi,ap2)
	       call sbdyi(natix,natiy,xati,yati,asa,aitype,
     &         xi,yi,as2)
                 lambda = cp2/freqx
		 call cnvnts(ap2,lambda,freqx,atiu,adBoW)
                 ap2 = adBoW
		 lambda = cs2/freqx
                 call cnvnts(as2,lambda,freqx,atiu,adBoW)
                 as2 = adBoW
                 call bdryr(rho1,rho2,ci,cp2,cs2,ap2,as2,theta,refl)
               else
                 print *, 'Unknown surface properties'
                 print *, '(neither H or N),'
                 print *, 'aborting calculations...'
                 stop
               end if 
             else
             print *, 'Unknown surface type (neither A,E,R or V),'
             print *, 'aborting calculations...'
             stop
             end if

            refldc = refldc*refl

c           Kill the ray if the reflection decay is too small: 

            if ( abs(refldc) .lt. 1.0e-6 ) ikill = 1 

          end if
c======================================================================
c         Ray below bottom?
           
          if (zi.gt.zbtyi) then
	  
	     jrefl = -1
             brefl = brefl + 1
             nrefl = nrefl + 1
	     
 	     if (nrefl.gt.nreflx) then
 	     print *, 'Reflection points: nrefl > ',nreflx
 	     print *, 'aborting calculations...'
 	     stop
 	     end if

	     call raybi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,la,lb,li)

             xi = li(1)
             yi = li(2)
	     zi = li(3)

             krefl(nrefl) = i+1
             
	     call fbdyi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,
     &       xi,yi,zbtyi,normal,d1,d2,d3,d4,d5)
 
c            Invert the normal at the bottom for reflection:

	     normal(1) = -normal(1)
	     normal(2) = -normal(2)
	     normal(3) = -normal(3)

c            Calculate bottom reflection:

             call reflct( normal, es, taur, theta )

c            Check if the ray is digging in beyond the bottom:

             call sbdyi(nbtyx,nbtyy,xbty,ybty,zbty,bitype,
     &       xi+ds*taur(1),yi+ds*taur(2),zbtyi)
	     
	     if ((zi+ds*taur(3)).gt.zbtyi) then
	     ikill = 1
	     end if

c            Bottom reflection => get the reflection coefficient
c            (kill the ray is the bottom is an absorver):

             if (btype.eq.'A') then
             refl = (0.0,0.0)
             ikill = 1
             elseif (btype.eq.'R') then
             refl = (1.0,0.0)
             elseif (btype.eq.'V') then
             refl = (-1.0,0.0)
             elseif (btype.eq.'E') then
               if (bptype.eq.'H') then
		  lambda = cpbty/freqx
                  call cnvnts(apbty,lambda,freqx,btyu,adBoW)
                  ap2  = adBoW
		  lambda = csbty/freqx
                  call cnvnts(asbty,lambda,freqx,btyu,adBoW)
		  as2  = adBoW
                  call bdryr(rho1,rhobty,ci,cpbty,csbty,ap2,as2,
     &            theta,refl)
               elseif (aptype.eq.'N') then
	       call sbdyi(nbtyx,nbtyy,xbty,ybty,rhob,bitype,
     &         xi,yi,rho2)
	       call sbdyi(nbtyx,nbtyy,xbty,ybty,cpb,bitype,
     &         xi,yi,cp2)
	       call sbdyi(nbtyx,nbtyy,xbty,ybty,csb,bitype,
     &         xi,yi,cs2)
	       call sbdyi(nbtyx,nbtyy,xbty,ybty,apb,bitype,
     &         xi,yi,ap2)
	       call sbdyi(nbtyx,nbtyy,xbty,ybty,asb,bitype,
     &         xi,yi,as2)
                 lambda = cp2/freqx
		 call cnvnts(ap2,lambda,freqx,btyu,adBoW)
                 ap2 = adBoW
		 lambda = cs2/freqx
                 call cnvnts(as2,lambda,freqx,btyu,adBoW)
                 as2 = adBoW
                 call bdryr(rho1,rho2,ci,cp2,cs2,ap2,as2,theta,refl)
               else
                 print *, 'Unknown bottom properties'
                 print *, '(neither H or N),'
                 print *, 'aborting calculations...'
                 stop
               end if 
             else
             print *, 'Unknown bottom type (neither A,E,R or V),'
             print *, 'aborting calculations...'
             stop
             end if

            refldc = refldc*refl

c           Kill the ray if the reflection decay is too small: 

            if ( abs(refldc) .lt. 1.0e-6 ) ikill = 1 

          end if
c======================================================================
	   
c         Update marching solution and function:
                    
          xi = li(1)
          yi = li(2)
	  zi = li(3)

          call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     &    cxxi,cyyi,czzi,cxyi,cxzi,cyzi)
     
          ynew(1) = xi
          ynew(2) = yi
	  ynew(3) = zi
          ynew(4) = sigmai*taur(1)
          ynew(5) = sigmai*taur(2)
	  ynew(6) = sigmai*taur(3)

          fnew(1) = taur(1)
          fnew(2) = taur(2)
	  fnew(3) = taur(3)
          fnew(4) = sxi
          fnew(5) = syi
	  fnew(6) = szi

          end if

c[====================================================================]

          xi     = ynew(1)
          yi     = ynew(2)
	  zi     = ynew(3)
	  
          x(i+1) = xi
          y(i+1) = yi
	  z(i+1) = zi
          
          es(1) = fnew(1)
          es(2) = fnew(2)
	  es(3) = fnew(3)
	  
	  mes(i+1,1) = es(1)
	  mes(i+1,2) = es(2)
	  mes(i+1,3) = es(3)

          call csvals(xi,yi,zi,ci,cc,sigmai,cxi,cyi,czi,sxi,syi,szi,
     &    cxxi,cyyi,czzi,cxyi,cxzi,cyzi)

          dx = x(i+1) - x(i)
	  dy = y(i+1) - y(i)
          dz = z(i+1) - z(i)
	  
          dri = sqrt( dx*dx + dy*dy )
          dsi = sqrt( dx*dx + dy*dy + dz*dz )
          
           tau(i+1) = tau(i) + dsi/ci
          cray(i+1) =    ci
             s(i+1) =   s(i) + dsi

          decay(i+1) = refldc
          irefl(i+1) = jrefl
          rayel(i+1) = atan2(dz,dri)
	  rayaz(i+1) = atan2(dy,dx )
	  
c         Detect returning rays by checking the turn variable 
c        (=accumulated azimuth); if turn > 180 degrees save the 
c        index of return and reset the turn variable:

	  turn = turn + abs( rayaz(i+1) - rayaz(i) )
	  
	  if (turn.gt.0.5*pi) then
	  iretrn = 1
	  turn = 0.0
	  end if
	  
	  jrefl = 0
            nrk = 0
 
          do j = 1,6
       
             yold(j) = ynew(j)
             fold(j) = fnew(j)
         
          end do

c-----------------------------------------------------------------------
c         Next one:

          i = i + 1
c-----------------------------------------------------------------------

          xmax = max(x(i),xmax)
          xmin = min(x(i),xmin)
          ymax = max(y(i),ymax)
          ymin = min(y(i),ymin)

c-----------------------------------------------------------------------
c         Prevent further calculations if there is no more space in 
c         the memory for the ray coordinates:

          if ( i.gt.np ) then

          print *, 'Ray step too small, number of points in ray'
          print *, 'coordinates > ',np,',' 
          print *, 'aborting calculations...'
          stop

          end if

c-----------------------------------------------------------------------
  
          go to 1000

       end if
       
       imax = i

c----------------------------------------------------------------------
c      Check of the ray exits the box (it can be killed before it):

       if ((x(imax).gt.xbox(2)).or.(x(imax).lt.xbox(1))) then 
       iout = 1
       end if
       
       if ((y(imax).gt.ybox(2)).or.(y(imax).lt.ybox(1))) then 
       iout = 1
       end if
       
c***********************************************************************
c      If the ray exits the box cut it at the box plane:
        
	if (iout.eq.1) then 
	 
        la(1) = x(imax-1)
        la(2) = y(imax-1)
        la(3) = z(imax-1)
        
        lb(1) = x( imax )
        lb(2) = y( imax )
        lb(3) = z( imax )

c=======================================================================
        if (x(imax).lt.xbox(1)) then
        
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(1)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    li(1) = xbox(1)
	    
c	        Verify that the final point is inside the box; 
c               if not do the cut at the lower or upper plane: 
 
	    if (li(2).lt.ybox(1)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(1)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    elseif  (li(2).gt.ybox(2)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(2)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(2)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    else
	    
	    end if
	    
        else if (x(imax).gt.xbox(2)) then
        
            p0(1) = xbox(2)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(2)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    li(1) = xbox(2)
	    
c	        Verify that the final point is inside the box; 
c               if not do the cut at the lower or upper plane: 
 
	    if (li(2).lt.ybox(1)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(1)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    elseif  (li(2).gt.ybox(2)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(2)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(2)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    else
	    
	    end if
	    	    
        else if (y(imax).lt.ybox(1)) then
        
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(1)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    li(2) = ybox(1)
	    
c	        Verify that the final point is inside the box; 
c               if not do the cut at the left or right plane: 
 
	    if (li(1).lt.xbox(1)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(1)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    elseif  (li(1).gt.xbox(2)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(2)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(2)
            p2(2) = ybox(2)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    else
	    
	    end if
	        
        else if (y(imax).gt.ybox(2)) then
        
            p0(1) = xbox(1)
            p0(2) = ybox(2)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(2)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    li(2) = ybox(2)
	    
c	        Verify that the final point is inside the box; 
c               if not do the cut at the left or right plane: 
 
	    if (li(1).lt.xbox(1)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(1)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(1)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(1)
            p2(2) = ybox(1)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    elseif (li(1).gt.xbox(2)) then
	    
            lb(1) = li(1)
            lb(2) = li(2)
            lb(3) = li(3)
	    
            p0(1) = xbox(2)
            p0(2) = ybox(1)
            p0(3) = 0.0
         
            p1(1) = xbox(2)
            p1(2) = ybox(2)
            p1(3) = 0.0
                
            p2(1) = xbox(2)
            p2(2) = ybox(2)
            p2(3) = zbtyx
	    
	    call lpintn(la,lb,p0,p1,p2,li)
	    
	    else
	    
	    end if
   
        else 
        
        end if
c=======================================================================

        x(imax) = li(1)
        y(imax) = li(2)
        z(imax) = li(3)
	
	dt  = tau(imax) - tau(imax-1)
	dsi =   s(imax) -   s(imax-1)
	dx  =   x(imax) -   x(imax-1)
	dy  =   y(imax) -   y(imax-1)
	dz  =   z(imax) -   z(imax-1)
	
	dsl = sqrt( dx*dx + dy*dy + dz*dz )
	
	tau(imax) = tau(imax-1) + dt*dsl/dsi
	  s(imax) =   s(imax-1) + dsl

	end if

c***********************************************************************
c       Get refraction points:
c***********************************************************************
	
        nrefr = 0

        do i = 2,imax-1
 
           prod = ( z(i+1)-z(i) )*( z(i)-z(i-1) )

           jbdry = abs( irefl(i) )
	   
 	   if ( ( prod.lt.0 ).and.(jbdry.ne.1) ) then
 
 	       if (nrefr.gt.nrefrx) then
	       print *, 'Refraction points: nrefr > ',nrefrx
               print *, 'refraction index invalid!'
               nrefr = 0
 	       end if
 
 	       nrefr = nrefr + 1
	       krefr(nrefr) = i
	       
 	   end if

        end do
	
c***********************************************************************

c***********************************************************************
c     Back to main:
c***********************************************************************

      return
      
      end
