       program traceo3d

c***********************************************************************
c
c      TRACEO3D Ray tracing program
c      Written by Orlando Camargo Rodriguez
C      Copyright (C) 2017 Orlando Camargo Rodriguez
c      orodrig@ualg.pt
c      Universidade do Algarve
c      Physics Department
c      Signal Processing Laboratory
c      Gambelas, Sex Jan 27 13:32:14 WET 2017
c
c***********************************************************************
c      Commands should be placed in columns 6-72!
c****&******************************************************************
c
c      License: The TRACEO3D Raytracing Model is released under the
c      Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License
c      http://creativecommons.org/licenses/by-nc-sa/3.0/
c
c***********************************************************************
c      Let's go:
c****&******************************************************************

c      Include global (common) variables:

       include 'global.for'

c      Define local variables:
       
       character(len=60) ctitle
       character(len=47) lnglne
       character(len=12) lognme
       
       integer iad,nthtas,nphi
       
       real*8 timei,timef
       
c-----------------------------------------------------------------------

       call cpu_time(timei)
       
       lnglne = '-----------------------------------------------'
       
       lognme = 'traceo3d.log'
c-----------------------------------------------------------------------

       open(unit=prtfil,file=lognme,status='unknown')
       write(prtfil,*) 'TRACEO3D ray tracing program'
       write(prtfil,*) 'Written by Orlando Camargo Rodriguez'
       write(prtfil,*) 'SiPLAB laboratory'
       write(prtfil,*) lnglne
       write(prtfil,*) 'INPUT:'

       call readin(ctitle,nthtas,nphi,iad)

       write(prtfil,*) ctitle
       write(prtfil,*) lnglne
       write(prtfil,*) 'OUTPUT:'

       if (catype.eq.'RCO') then

           write(prtfil,*) 'Ray coordinates'

           call calrco(ctitle,nthtas,nphi)
	   
       elseif (catype.eq.'ARI') then

           write(prtfil,*) 'All ray information'

           call calari(ctitle,nthtas,nphi)
	   
       else if (catype.eq.'CPR') then

           write(prtfil,*) 'Coherent acoustic pressure'

           call calcpr(ctitle,nthtas,nphi,iad)

       else if (catype.eq.'PVL') then

           write(prtfil,*) 'Particle velocity'

           call calpvl(ctitle,nthtas,nphi,iad)

       else if (catype.eq.'PAV') then

           write(prtfil,*) 'Pressure and particle velocity'
 
           call calpav(ctitle,nthtas,nphi,iad)
 
       else if (catype.eq.'EIG') then

           write(prtfil,*) 'Eigenrays'
 
           call caleig(ctitle,nthtas,nphi,iad)
	   
       else if (catype.eq.'AAD') then

           write(prtfil,*) 'Amplitudes and delays'
 
           call calaad(ctitle,nthtas,nphi,iad)

       else

           write(prtfil,*) 'Unknown output option,'
	   write(prtfil,*) 'aborting calculations...'
           stop

       end if 
       
       call cpu_time(timef)
       
       write(prtfil,*) lnglne
       write(prtfil,*) 'done.'
       write(prtfil,*) ' '
       write(prtfil,*) 'CPU time:', timef-timei, ' seconds'
       
       close(prtfil)

c***********************************************************************
c      C'est fini!
c***********************************************************************

       end
